import re

import pandas as pd
from pandas import DataFrame

from other.common import write_file
from other.common.file.list import get_files_in_folder, get_all_folders_in_folder
from other.etl import VioUser

rank_cacher = {}
regex_pattern = '24_01_2021\/(.*?)\/(.*?)\/(.+)\/'
input_folder = 'etl/user/vio/input_data/24_01_2021'
teacher_output = 'etl/user/vio/output_data/teacher.csv'
student_output = 'etl/user/vio/output_data/student.csv'
split_by_school = True


def create_student_from_df_row(df: DataFrame, row_num: int, file_name: str):
    student = VioUser()
    student.type = 'Student'
    student.province_name = 'Hà Nội'
    file_name = file_name.replace('\\', '/')
    regex_rs = re.findall(regex_pattern + '.+Khối([1-9])?', file_name)
    if regex_rs:
        regex_rs = regex_rs[0]
        student.district_name = regex_rs[0]
        student.school_level = regex_rs[1]
        student.school_name = regex_rs[2]
        student.student_class = regex_rs[3]

    student.vio_username = df['Tên đăng nhập'].tolist()[row_num]
    student.vio_password = df['Mật khẩu'].tolist()[row_num]
    student.full_name = df['Họ tên'].tolist()[row_num]
    student.ms_username = df['Tài khoản MS Teams'].tolist()[row_num]
    student.ms_password = df['Mật khẩu MS Teams'].tolist()[row_num]
    student.gender = df['Giới tính'].tolist()[row_num]
    student.dob = df['Ngày sinh'].tolist()[row_num]
    student.address = df['Địa chỉ'].tolist()[row_num]
    student.student_class_name = df['Lớp'].tolist()[row_num]
    student.student_code = df['Mã học sinh'].tolist()[row_num]
    student.parent_email = df['Email phụ huynh'].tolist()[row_num]
    student.parent_phone_number = df['Số điện thoại phụ huynh'].tolist()[row_num]
    student.vio_parent_username = df['Tài khoản phụ huynh thứ 1'].tolist()[row_num]
    student.vio_parent_password = df['Mật khẩu của tài khoản phụ huynh thứ 1'].tolist()[row_num]
    student.vio_parent_username_2 = df['Tài khoản phụ huynh thứ 2'].tolist()[row_num]
    student.vio_parent_password_2 = df['Mật khẩu của tài khoản phụ huynh thứ 2'].tolist()[row_num]
    if student.vio_username in rank_cacher.keys():
        contest = rank_cacher[student.vio_username]
        student.contest_count_correct_answer = contest['contest_count_correct_answer']
        student.contest_count_wrong_answer = contest['contest_count_wrong_answer']
        student.contest_duration = contest['contest_duration']
        student.contest_rank = contest['contest_rank']
        del rank_cacher[student.vio_username]
    return student


def create_teacher_from_df_row(df: DataFrame, row_num: int, file_name: str):
    teacher = VioUser()
    teacher.type = 'Teacher'
    teacher.province_name = 'Hà Nội'
    file_name = file_name.replace('\\', '/')
    regex_rs = re.findall(regex_pattern, file_name)
    if regex_rs:
        regex_rs = regex_rs[0]
        teacher.district_name = regex_rs[0]
        teacher.school_level = regex_rs[1]
        teacher.school_name = regex_rs[2]

    teacher.vio_username = df['Tên đăng nhập'].tolist()[row_num]
    teacher.vio_password = df['Mật khẩu'].tolist()[row_num]
    teacher.full_name = df['Họ tên'].tolist()[row_num]
    teacher.ms_username = df['Tên đăng nhập MS Teams'].tolist()[row_num]
    teacher.ms_password = df['Mật khẩu Ms Teams'].tolist()[row_num]
    teacher.gender = df['Giới tính'].tolist()[row_num]
    teacher.dob = df['Ngày sinh'].tolist()[row_num]
    teacher.address = df['Địa chỉ'].tolist()[row_num]
    teacher.email = df['Email'].tolist()[row_num]
    teacher.phone_number = df['Số điện thoại'].tolist()[row_num]
    teacher.identity_card_number = df['Số CMNTD'].tolist()[row_num]
    teacher.teacher_subject = str(df['Môn'].tolist()[row_num]).replace(',', '-')
    teacher.teacher_class_name = str(df['Lớp'].tolist()[row_num]).replace(',', '-')
    return teacher


def _load_df_from_xlsx_file(file_name: str):
    xls = pd.ExcelFile(file_name)
    df = xls.parse(0)  # first sheet
    df.columns = df.iloc[0]
    df = df[1:]
    return df


def _insert_to_cache_from_rank_df(df: DataFrame, file_name: str):
    total_row = len(df)
    for row_num in range(total_row):
        rank_cacher.update({
            df['Tên đăng nhập'].tolist()[row_num]: {
                'contest_count_wrong_answer': df['Số câu sai'].tolist()[row_num],
                'contest_count_correct_answer': df['Số câu đúng'].tolist()[row_num],
                'contest_duration': df['Tổng số giây suy nghĩ'].tolist()[row_num],
                'contest_rank': df['Thứ hạng'].tolist()[row_num],
            }
        })


def _load_student_from_df(df: DataFrame, file_name: str):
    total_row = len(df)
    list_student = []
    for row_num in range(total_row):
        student = create_student_from_df_row(df=df, row_num=row_num, file_name=file_name)
        list_student.append(student)
    return list_student


def _load_teacher_from_df(df: DataFrame, file_name: str):
    total_row = len(df)
    list_teacher = []
    for row_num in range(total_row):
        teacher = create_teacher_from_df_row(df=df, row_num=row_num, file_name=file_name)
        list_teacher.append(teacher)
    return list_teacher


def _process_tecacher_xlsx_in_partition(partition_path: str):
    teacher_xlsx_files = get_files_in_folder(root_folder=partition_path)

    if split_by_school:
        teacher_output_by_parttion = teacher_output.replace('.csv',
                                                            partition_path
                                                            .replace(input_folder, '-')
                                                            .replace('/', '-')
                                                            + '.csv')
        write_file(file_path=teacher_output_by_parttion, content=VioUser.get_header_str() + '\n')
    for teacher_xlsx_file in teacher_xlsx_files:
        if any(e in teacher_xlsx_file for e in ['xếp', 'học']):
            continue
        print(teacher_xlsx_file)
        df = _load_df_from_xlsx_file(file_name=teacher_xlsx_file)
        list_teacher = _load_teacher_from_df(df=df, file_name=teacher_xlsx_file)
        write_file(file_path=teacher_output_by_parttion if split_by_school else teacher_output,
                   write_mode='a',
                   content=list_teacher,
                   is_list=True)


def _process_student_xlsx_in_partition(partition_path: str):
    student_xlsx_files = get_files_in_folder(root_folder=partition_path)
    rank_xlsx_files = get_files_in_folder(root_folder=partition_path)
    for rank_xlsx_file in rank_xlsx_files:
        if any(e in rank_xlsx_file for e in ['học', 'giáo', 'giao']):
            continue
        rank_df = _load_df_from_xlsx_file(file_name=rank_xlsx_file)
        _insert_to_cache_from_rank_df(df=rank_df, file_name=rank_xlsx_file)

    if split_by_school:
        student_output_by_parttion = student_output.replace('.csv',
                                                            partition_path
                                                            .replace(input_folder, '-')
                                                            .replace('/', '-')
                                                            + '.csv')
        write_file(file_path=student_output_by_parttion, content=VioUser.get_header_str() + '\n')
    for student_xlsx_file in student_xlsx_files:
        if any(e in student_xlsx_file for e in ['xếp', 'giáo', 'giao']):
            continue
        print(student_xlsx_file)
        df = _load_df_from_xlsx_file(file_name=student_xlsx_file)
        list_student = _load_student_from_df(df=df, file_name=student_xlsx_file)
        write_file(file_path=student_output_by_parttion if split_by_school else student_output,
                   write_mode='a',
                   content=list_student,
                   is_list=True)


def _process_partition(partition_path: str):
    _process_tecacher_xlsx_in_partition(partition_path=partition_path)
    _process_student_xlsx_in_partition(partition_path=partition_path)


def process():
    list_partition_path = get_all_folders_in_folder(root_folder=input_folder)
    if not split_by_school:
        write_file(file_path=teacher_output, content=VioUser.get_header_str() + '\n')
        write_file(file_path=student_output, content=VioUser.get_header_str() + '\n')
    for partition_path in list_partition_path:
        _process_partition(partition_path)


process()



import glob
import os
import urllib.request

from ucode.helpers.clog import CLog


def download_all_scratch_submission(folder):
    sub_folders = glob.glob(folder + "/*")
    print(sub_folders)
    has_scratch_online = []
    for sub_folder in sub_folders:
        if not os.path.isdir(sub_folder):
            continue
        print(sub_folder)
        files = glob.glob(sub_folder + "/*.txt")
        for file in files:
            path, name = os.path.split(file)
            name, extension = os.path.splitext(name)
            with open(file, "r") as f:
                url = f.read()
                downloaded_file = os.path.join(path, name + ".sb3")
                print(url, downloaded_file)
                if "scratch.mit.edu" in url:
                    downloaded_file = os.path.join(path, name + "-ERROR-scratch-online.err")
                    CLog.error(f"Scratch online: {downloaded_file}")
                    _, student = os.path.split(path)
                    has_scratch_online.append(student)
                urllib.request.urlretrieve(url, downloaded_file)
    print(len(has_scratch_online))
    print(*sorted(has_scratch_online), sep="\n")
    has_scratch_online = set(has_scratch_online)
    print((len(has_scratch_online)))
    print(*sorted(has_scratch_online), sep="\n")


if __name__ == "__main__":
    # folder = r"D:\Tin học tre\So khao Quoc gia 2021\Ket_qua_bang_A_final"
    folder = r"D:\Tin học tre\So khao Quoc gia 2021\test"
    download_all_scratch_submission(folder)


# coding=utf-8
import logging
import os
import stat
import subprocess
from typing import List


__author__ = 'ThucNC'
_logger = logging.getLogger(__name__)


class ShellHelper:
    @staticmethod
    def detect_executable(command):
        """
        Try to find an `command` executable. It first looks in
        ./command/, then the local directory, then in a relative path
        from the file that contains the Sandbox module, then in the
        system paths.

        return (string): the path to a valid (hopefully) command.

        """
        paths = [os.path.join('', command, command),
                 os.path.join('', command)]
        if '__file__' in globals():
            paths += [os.path.abspath(os.path.join(
                os.path.dirname(__file__),
                '..', '..', command, command))]
        paths += [command]
        print(paths)
        for path in paths:
            # Consider only non-directory, executable files with SUID flag on.
            if os.path.exists(path) \
                    and not os.path.isdir(path) \
                    and os.access(path, os.X_OK):
                st = os.stat(path)
                if st.st_mode & stat.S_ISUID != 0:
                    return path

        # As default, return self.exec_name alone, that means that
        # system path is used.
        return paths[-1]

    @staticmethod
    def execute(commands: List[str], strip_output=True, raise_exception=False, timeout=60, input=None):
        process = subprocess.run(commands, stdout=subprocess.PIPE,
                                 universal_newlines=True, stderr=subprocess.PIPE,
                                 timeout=timeout, input=input)
        # print(process)
        if process.returncode != 0:
            msg = f"Failed to run command {commands} with error: {process.returncode}, stderr: \n {process.stderr} "
            if raise_exception:
                raise Exception(msg)
            else:
                _logger.error(msg)

        res = process.stdout
        if strip_output:
            res = res.strip()
        return res


if __name__ == "__main__":
    print(ShellHelper.detect_executable("python"))



import glob
import os

from ucode.helpers.clog import CLog
from ucode.helpers.misc import findfiles
from ucode.models.problem import Problem
from ucode.services.dsa.problem_service import ProblemService
from ucode.services.testcase.testcase_service import TestcaseService
from unidecode import unidecode


class FreeContest:
    @staticmethod
    def load_all_free_contest(folder):
        folders = glob.glob(os.path.join(folder, "*"))
        contests = []
        for f in folders:
            contest_name = os.path.basename(f)
            contests.append((contest_name, FreeContest.load_free_contest(f)))
        CLog.info(f"Loaded {len(contests)} contest(s)!")
        return contests

    @staticmethod
    def load_free_contest(folder, save_to_folder=None, difficulty=3):
        contest_folders = {
            "statement": "",
            "solution": "",
            "testcase": "",
            "editorial": ""
        }
        contest_name = os.path.basename(folder)
        CLog.info(f"Loading contest `{contest_name}` in {folder}")
        folders = glob.glob(os.path.join(folder, "*"))
        for f in folders:
            fname = unidecode(os.path.basename(f).lower())
            print(fname)
            if "bai tap" in fname or "de bai" in fname:
                contest_folders['statement'] = f
            elif "test" in fname:
                contest_folders['testcase'] = f
            elif "bai giai" in fname:
                contest_folders['solution'] = f
            elif "loi giai" in fname:
                contest_folders['editorial'] = f
        # print(contest_folders)
        if not contest_folders['statement']:
            CLog.error(f"No statement folder found!")
            return None
        if not contest_folders['testcase']:
            CLog.error(f"No testcase folder found!")
            return None
        statement_files = findfiles("*.pdf", contest_folders['statement'])
        problem_codes = []
        for statement_file in statement_files:
            problem_name = os.path.basename(statement_file)
            problem_codes.append(problem_name[:problem_name.find(".")])
        print("problem_codes:", problem_codes)

        problems = []
        for problem_code in problem_codes:
            problem = FreeContest.load_problem(problem_code, contest_folders, contest_name, difficulty=difficulty)
            problems.append(problem)
            # print(problem_code, problem.name)
            if problem_code != problem.name:
                CLog.error(f"Please double check problem name: {problem_code}, {problem.name}")
                raise Exception(f"Please double check problem name: {problem_code}, {problem.name}")
        # TODO: test with checker
        CLog.info(f"Loading contest from folder: {folder}")
        CLog.info(f"{len(problems)} problem(s) found!")
        if save_to_folder:
            if not os.path.exists(save_to_folder) or not os.path.isdir(save_to_folder):
                CLog.error(f"Invalid output folder: {save_to_folder}")
            else:
                base_folder = os.path.join(save_to_folder, contest_name)
                for problem in problems:
                    ProblemService.save(problem, base_folder=base_folder, overwrite=True)
        return problems

    @staticmethod
    def load_problem(problem_code, contest_folders, contest_name, difficulty=3):
        problem = Problem()
        problem.name = problem.code = problem_code
        problem.difficulty = difficulty
        problem.src_name = "FreeContest"
        problem.src_url = contest_name
        statement_file = os.path.join(contest_folders['statement'], f"{problem_code}.md")
        if os.path.exists(statement_file):
            CLog.info(f"Found .md statement file at {statement_file}")
            ProblemService.parse_statement_file(statement_file, problem)
        else:
            statement_file = os.path.join(contest_folders['statement'], f"{problem_code}.pdf")
            problem.statement = statement_file
            problem.statement_format = "pdf"
            CLog.error(f".pdf statement file is not supported yet: {statement_file}")
            return None

        test_file = os.path.join(contest_folders['testcase'], f"{problem_code}.zip")
        if not os.path.exists(statement_file):
            CLog.error(f"Testcases not found for problem `{problem_code}` at: {test_file}")
            return None
        problem.testcases = TestcaseService.load_testcases(test_file, "cms")
        return problem


if __name__ == "__main__":
    free_contests = """Free Contest 35
Free Contest 36
Free Contest 37
Free Contest 38
Free Contest 39
Free Contest 40
Free Contest 41
Free Contest 42
Free Contest 44
Free Contest 45
Free Contest 46
Free Contest 47
Free Contest 48
Free Contest 49
Free Contest 51
Free Contest 52
Free Contest 53
Free Contest 54
Free Contest 55
Free Contest 56
Free Contest 57
Free Contest 58
Free Contest 59
Free Contest 60
Free Contest 61
Free Contest 62
Free Contest 63
Free Contest 64
Free Contest 65
Free Contest 80
Free contest 94
Free contest 95
Free contest 96
Free contest 97
Free contest 98
Free contest 99
Free contest 100
Free contest 101
Free contest 102
Free contest 103
Free contest 104
Free contest 105
Free contest 106
Free contest 107
Free contest 108
Free contest 109
Free contest 110
Free contest 111
Free contest 112
Free contest 113
Free contest 114
Free contest 115
Free contest 116
Free contest 117
Free contest 118
Free contest 119
Free contest 120
Free Contest 121"""

    beginner_contests = """Beginner Free Contest 01 (15-10-2018)
Beginner Free Contest 02 (14-11-2018)
Beginner Free Contest 03 (20-01-2019)
Beginner Free Contest 04 (31-01-2019)
Beginner Free Contest 05 (03-03-2019)
Beginner Free Contest 06 (17-04-2019)
Beginner Free Contest 07 (26-05-2019)
Beginner Free Contest 08 (12-06-2019)
Beginner Free Contest 10 (02-08-2019)
Beginner Free Contest 11 (01-09-2019)
Beginner Free Contest 12 (06-10-2019)
Beginner Free Contest 13 (30-10-2019)
Beginner Free Contest 14 (02-01-2020)
Beginner Free Contest 15 (12-01-2020)
Beginner Free Contest 16 (23-01-2020)
Beginner Free Contest 17 (25-04-2020)
Beginner Free Contest 18 (23-05-2020)
Beginner Free Contest 19 (16-06-2020)
Beginner Free Contest 20 (07-07-2020)
Beginner Free Contest 22 (15-08-2020)
Beginner Free Contest 23 (15-09-2020)
Beginner Free Contest 24 (20-10-2020)
Beginner Free Contest 25 (17-11-2020)
Beginner Free Contest 26 (22-12-2020)
Beginner Free Contest 27 (15-01-2021)
Beginner Free Contest 28 (27-03-2021)
Beginner Free Contest 29 (24-04-2021)"""
    # contest_folder = "D:\\projects\\freecontest\\beginner\\Beginner Free Contest 29 (24-04-2021)"
    out_folder = "D:\\projects\\freecontest\\beginner_formatted"
    # out_folder = "D:\\projects\\freecontest\\formatted"

    # contest_folder = "D:\\projects\\freecontest\\beginner\\Beginner Free Contest 27 (15-01-2021)"
    # out_folder = "D:\\projects\\freecontest\\formatted"
    # out_folder = None
    free_contests = "Free Contest 98"
    # free_contests = "Beginner Free Contest 27 (15-01-2021)"
    for folder in free_contests.split("\n"):
        # contest_folder = f"D:\\projects\\freecontest\\beginner\\{folder.strip()}"
        contest_folder = f"D:\\projects\\freecontest\\free_contest\\{folder.strip()}"
        # out_folder = "D:\\projects\\freecontest\\formatted"
        out_folder = "D:\\projects\\freecontest\\formatted"
        # out_folder = None
        problems = FreeContest.load_free_contest(contest_folder, save_to_folder=out_folder, difficulty=6)
    # for problem in problems:
    #     print(problem)
    print(f"Loaded {len(problems)} problems!")

    # all_contest_folder = "D:\\projects\\freecontest\\free_contest"
    # contests = FreeContest.load_all_free_contest(all_contest_folder)
    # for contest in contests:
    #     print(contest[0], len(contest[1]))




import json

import redis

from other.common.file.io import remove_if_exists, write_file

import mysql.connector

_HOST = 'localhost'
_PORT = '3311'
_USER = 'ucode'
_PASSWORD = 'ucode1234'
_DATABASE = 'ucode_19_02'


def get_ucode_user_token():
    mydb = mysql.connector.connect(
        host=_HOST,
        port=_PORT,
        user=_USER,
        password=_PASSWORD,
        database=_DATABASE
    )
    mycursor = mydb.cursor(dictionary=True)
    mycursor.execute(
        '''
select user_id, token from user_token where id in (select max(id) from user_token where expired_at > now() and is_deleted = 0 group by user_id) limit 1000
        '''
    )
    users = mycursor.fetchall()
    tokens = [user['token'] for user in users]
    remove_if_exists('other/common/database/variable/ucode_student_user_token.py')
    write_file('other/common/database/variable/ucode_student_user_token.py',
               'UCODE_STUDENT_USER_TOKEN = ' + json.dumps(tokens, indent=2))
    mycursor.close()
    mydb.close()
    redisClient = redis.StrictRedis(host='localhost',
                                    port=6379,
                                    db=0)
    key = 'tokens'
    redisClient.lpush(key, *tokens)
    return tokens



# coding=utf-8
import logging

__author__ = 'ThucNC'

import re

_logger = logging.getLogger(__name__)


def find_section(pattern, lines, start_index=0, once=False, until_pattern="^#", skip_section_header=True):
    """
    Find a section in list of lines of text, start with line match `pattern`,
    until the next line that matches `until_pattern`
    @param skip_section_header:
    @param until_pattern:
    @param pattern:
    @param lines:
    @param start_index:
    @param once:
    @return: list of starting indices for each section found, and dict of their content
    """
    indices = []
    content = {}
    for i in range(start_index, len(lines)):
        if re.match(pattern, lines[i], re.I):
            indices.append(i)
            content.setdefault(i, [])
            if not skip_section_header:
                content[i].append(lines[i])
            for j in range(i+1, len(lines)):
                if re.match(until_pattern, lines[j], re.I):
                    break
                content[i].append(lines[j])
            if once:
                break

    return indices, content



import glob
import json
import os
import time
from builtins import staticmethod
from zipfile import ZipFile, is_zipfile

from jinja2 import Template
from ucode.helpers.clog import CLog
from ucode.helpers.misc import dos2unix, findfiles
from ucode.helpers.shell_helper import ShellHelper
from ucode.models.problem import Problem, TestCase


class TestcaseService:
    @staticmethod
    def remove_carriage_return(testcases):
        for testcase in testcases:
            testcase.input = testcase.input.replace("\r\n", "\n")
            testcase.output = testcase.output.replace("\r\n", "\n")
        return testcases

    @staticmethod
    def load_testcases(path: str):
        """

        @param path:    .txt / .zip file for 'ucode' format
                        .zip file / folder for 'cms', 'themis', 'hackerrank'
                        .zip file if format is 'cms',
                        folder if format is 'themis'
        @param format: ucode / cms / themis / hackerrank
        @return: List[Testcase]
        """

        path = os.path.abspath(path)
        if not os.path.exists(path):
            CLog.error(f"Path not existed: {path}")
            return None

        if os.path.isfile(path) and not is_zipfile(path):
            CLog.info(f"Loading testcases from single file (ucode format): {path}")
            return TestcaseService.remove_carriage_return(TestcaseService.read_ucode_testcases_from_file(path))

        if is_zipfile(path):
            CLog.info(f"Loading testcases from zipfile: {path}")
            with ZipFile(path, 'r') as myzip:
                format, testcase_files = TestcaseService.get_testcase_files_list_from_zip(path, zip_file=myzip)
                CLog.info(f"Detected testcase format `{format}` in zip file: `{path}`")
                if format == "ucode":
                    inside_archive_file = testcase_files[0]
                    CLog.info(f"Loading testcases (ucode format) from file: {path}\\{inside_archive_file}")
                    return TestcaseService.remove_carriage_return(TestcaseService.read_ucode_testcases_from_file(
                        path, inside_archive_file=inside_archive_file))
                else: # format == "themis" / "cms" / "themis" / "hackerrank"
                    return TestcaseService.remove_carriage_return(TestcaseService.read_testcases_from_zip_file(
                        path, inside_archive_files=testcase_files))

        else: # test folder
            format, testcase_files = TestcaseService.get_testcase_files_list_from_folder(path)
            CLog.info(f"Detected testcase format `{format}` in folder: `{path}`")
            if format == "ucode":
                test_file = testcase_files[0]
                CLog.info(f"Loading testcases (ucode format) from file: {test_file}")
                return TestcaseService.remove_carriage_return(TestcaseService.read_ucode_testcases_from_file(test_file))
            elif format == "themis":
                return TestcaseService.remove_carriage_return(TestcaseService.read_themis_testcases_from_folder(path))
            elif format == "hackerrank":
                pass

        # if format == 'ucode':
        #     return TestcaseService.read_ucode_testcases_from_file(path)
        # elif format == 'themis':
        #     return TestcaseService.read_themis_testcases_from_folder(path)
        # elif format == "cms":
        #     return TestcaseService.read_cms_testcases_from_zip_file(path)
        # else:
        #     CLog.error("Testcase format not supported: " + format)

    @staticmethod
    def find_name_from_list_ignore_case(filename, name_list):
        for name in name_list:
            if name.lower() == filename.lower():
                return name
        return None

    @staticmethod
    def get_testcase_files_list_from_folder(path):
        all_files = sorted(glob.glob(os.path.join(path, "*")))
        folders = [f for f in all_files if os.path.isdir(f)]
        print("Folder", folders)
        has_input = has_output = False
        has_folder = len(folders) > 0
        for folder in folders:
            if folder.endswith("input"):
                has_input = True
            if folder.endswith("input"):
                has_output = True
        if has_input and has_output:
            format = "hackerrank"
            CLog.error(f"Reading hackerrank testcases from folder is not supported yet")
        elif has_folder:
            format = "themis"
        else:
            format = "cms"
            CLog.error(f"Reading CMS testcases from folder is not supported yet")

        if not all_files:
            CLog.error(f"No files found in {path}")
            return None

        if len(all_files) < 2 or "testcases.txt" in all_files:
            format = "ucode"
            return format, all_files

        print(all_files)

        return format, all_files

    @staticmethod
    def get_testcase_files_list_from_zip(path, zip_file):
        all_files = sorted(zip_file.namelist())
        has_input = has_output = False
        folder_count = 0
        # prefix =
        # print("all_files", *all_files, sep="\n")
        for file in all_files:
            if file.endswith("/"):
                folder_count += 1
            first_slash = file.find("/")
            if file.lower().startswith("input/") \
                    or first_slash > 0 and file.lower()[first_slash+1:].startswith("input/"):
                has_input = True
            if file.lower().startswith("output/") \
                    or first_slash > 0 and file.lower()[first_slash + 1:].startswith("output/"):
                has_output = True
        if has_input and has_output:
            format = "hackerrank"
            inputs = [f for f in all_files if (f.startswith("input/") or "/input/" in f) and not f.endswith("/")]
            test_files = []
            for input_file in inputs:
                output_file = input_file.replace("input", "output").replace(".in", ".out")
                test_files.append((input_file, output_file))
            print(test_files)
            return format, test_files
        elif folder_count > 1:
            format = "themis"
            test_files = []
            inputs = [f for f in all_files if f.lower().endswith(".inp") or f.lower().endswith(".in")]
            for input_file in inputs:
                if input_file.lower().endswith(".inp"):
                    output_file = TestcaseService\
                        .find_name_from_list_ignore_case(input_file.lower().replace(".inp", ".out"), all_files)
                elif input_file.lower().endswith(".in"):
                    output_file = TestcaseService \
                        .find_name_from_list_ignore_case(input_file.lower().replace(".in", ".out"), all_files)
                    if not output_file:
                        output_file = TestcaseService \
                            .find_name_from_list_ignore_case(input_file.lower().replace(".in", ".ou"), all_files)

                test_files.append((input_file, output_file))
            print(test_files)
            return format, test_files
        else:
            format = "cms"
            # format 1: input.xxx & output.xxx
            # format 2: x.in & x.ou
            # format 3: x.in & x.ok
            # format 4: x.in & x.ans
            # format 5: inputxx.inp & inputxx.out
            # format 6: x.inp & x.out
            # format 7: inputxx.txt & inputxx.txt
            test_files = []
            input_files = [f for f in all_files if f.lower().endswith(".inp")]
            if not input_files:
                input_files = [f for f in all_files if f.lower().endswith(".in")]
            if not input_files:
                input_files = [f for f in all_files if f.lower().startswith("input") or "/input" in f.lower()]
            input_files.sort(key=lambda s: int("".join([c for c in s if "0" <= c <= "9"])))
            # print("input files", input_files)
            for input_file in input_files:
                if input_file.lower().endswith(".inp"):
                    output_file = TestcaseService.\
                        find_name_from_list_ignore_case(input_file.replace(".inp", ".out"), all_files)
                elif input_file.lower().startswith("input") or "/input" in input_file.lower():
                    output_file = TestcaseService.\
                        find_name_from_list_ignore_case(input_file.lower().replace("input", "output"), all_files)
                    if not output_file:
                        output_file = TestcaseService. \
                            find_name_from_list_ignore_case(input_file.lower().replace("input", "ouput"), all_files)
                else:
                    output_file = TestcaseService. \
                            find_name_from_list_ignore_case(input_file.lower().replace(".in", ".ou"), all_files)
                    if not output_file:
                        output_file = TestcaseService. \
                            find_name_from_list_ignore_case(input_file.lower().replace(".in", ".ok"), all_files)
                    if not output_file:
                        output_file = TestcaseService. \
                            find_name_from_list_ignore_case(input_file.lower().replace(".in", ".ans"), all_files)
                    if not output_file:
                        output_file = TestcaseService. \
                            find_name_from_list_ignore_case(input_file.lower().replace(".in", ".out"), all_files)
                # CLog.info(f"Reading a testcase in {input_file}, {output_file}")
                test_files.append((input_file, output_file))
            print(test_files)
            return format, test_files

        if not all_files:
            CLog.error(f"No files found in {path}")
            return None

        if len(all_files) < 2 or "testcases.txt" in all_files:
            format = "ucode"
            return format, all_files

        print(all_files)

        return format, all_files

    @staticmethod
    def read_testcases_from_zip_file(zipfile, inside_archive_files):
        testcases = []
        CLog.info(f"Loading testcases from .zip file `{os.path.abspath(zipfile)}`")

        with ZipFile(zipfile, 'r') as myzip:
            for (input_file, output_file) in inside_archive_files:
                fi = myzip.open(input_file)
                fo = myzip.open(output_file)
                _input = TestcaseService.read_file(input_file, fi, myzip)
                _output = TestcaseService.read_file(output_file, fo, myzip)
                testcases.append(TestCase(input=_input, output=_output))

        CLog.info(f"DONE: {len(testcases)} testcases loaded!")
        return testcases

    @staticmethod
    def read_themis_testcases_from_folder(path):
        testcases = []

        test_folders = glob.glob(os.path.join(path, "Test*"))
        for test_folder in test_folders:
            # test_folder = os.path.abspath(test_folder)
            if not os.path.isdir(test_folder):
                continue
            # print(test_folder)

            input_files = findfiles("*.INP", test_folder)
            output_files = findfiles("*.OUT", test_folder)
            # print(input_files, len(input_files))
            # print(output_files, len(output_files))
            if len(input_files) != 1 or len(output_files) != 1:
                CLog.error(f"Invalid testcase folder `{os.path.abspath(test_folder)}`")
                continue
            input_file = input_files[0]
            output_file = output_files[0]

            # print(input_file, output_file)
            with open(input_file, 'r', encoding="utf-8") as fi:
                _input = fi.read()
            with open(output_file, 'r', encoding="utf-8") as fo:
                _output = fo.read()
            testcases.append(TestCase(input=_input, output=_output))
        return testcases

    @staticmethod
    def read_ucode_testcases_from_file(testcase_file, inside_archive_file=None):
        count = 0
        inputi = ""
        outputi = ""
        is_output = False

        testcases = []
        name = ""
        if inside_archive_file:
            with ZipFile(testcase_file, 'r') as myzip:
                fi = myzip.open(inside_archive_file)
                lines = fi.read().decode("utf-8").split("\n")
        else:
            with open(testcase_file, 'r', encoding="utf-8") as fi:
                lines = fi.readlines()

        for line in lines:
            if line.startswith("###"):

                if count > 0:
                    # testcases.append({'input': inputi.rstrip(), 'output': outputi.rstrip(), 'name': name})
                    testcases.append(TestCase(input=inputi.rstrip(), output=outputi.rstrip(), name=name))

                name = line[3:].strip()
                # print("N:", name)
                count += 1

                is_output = False

                inputi = outputi = ""

                continue
            elif line.startswith("---"):
                is_output = True
            else:
                if is_output:
                    outputi += line
                else:
                    inputi += line
        if inputi.rstrip() or outputi.rstrip():
            # testcases.append({'input': inputi.rstrip(), 'output': outputi.rstrip(), 'name': name})
            testcases.append(TestCase(input=inputi.rstrip(), output=outputi.rstrip(), name=name))

        return testcases

    @staticmethod
    def execute(source_code_file: str, _input: str = "", raise_exception=True, timeout=60):
        """

        @param timeout:
        @param raise_exception:
        @param source_code_file:
        @param _input:
        @return: output, compile_time, execute_time
        """

        folder, file_name = os.path.split(source_code_file)
        file_base_name, file_ext = os.path.splitext(file_name)

        if file_ext == ".py":
            compile_needed = False
        else:
            compile_needed = True

        t0 = time.time()
        compile_time = -1

        owd = os.getcwd()
        os.chdir(folder)

        if compile_needed:
            if file_ext != ".cpp":
                CLog.error(f"Language not supported: `{file_ext}`")
                os.chdir(owd)
                return
            # C:\\Program Files\\CodeBlocks\\MinGW\\bin\\g++
            compile_cmd = ["g++.exe", "-std=c++14",
                           file_name,
                           "-pipe", f"-o{file_base_name}.exe"]
            ShellHelper.execute(compile_cmd, timeout=timeout)
            compile_time = time.time() - t0

        if file_ext == ".py":
            run_cmd = ["python", file_name]
        else:
            run_cmd = [f"{file_base_name}.exe"]

        t0 = time.time()
        output = ShellHelper.execute(run_cmd, input=_input, raise_exception=raise_exception)
        execute_time = time.time() - t0
        os.chdir(owd)
        return output, compile_time, execute_time

    @staticmethod
    def save_testcases(problem_folder, problem: Problem, format="ucode"):
        """

        @param problem_folder:
        @param problem:
        @param format: ucode | themis | both
        @return:
        """
        template_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), '../dsa/templates')

        if format == "ucode":
            CLog.info("Writing testcases in ucode format")
            testcases = problem.testcases
            if testcases:
                with open(os.path.join(template_path, 'testcases.txt.j2'), encoding="utf-8") as file_:
                    template = Template(file_.read())
                    for i, t in enumerate(testcases):
                        t.name = "Test #%02d" % (i + 1)
                    content = template.render(testcases=testcases)
                    testcase_path = os.path.join(problem_folder, "testcases.txt")
                    f = open(testcase_path, 'w', encoding="utf-8", newline='')
                    f.write(dos2unix(content))
                    f.close()
                    CLog.info(f"Testcases saved to '{testcase_path}'")

            if problem.stock_testcases:
                with open(os.path.join(template_path, 'testcases.txt.j2'), encoding="utf-8") as file_:
                    template = Template(file_.read())
                    for i, t in enumerate(problem.stock_testcases):
                        t.name = "Test #%02d" % (i + 1)
                    content = template.render(testcases=problem.stock_testcases)
                    testcase_path = os.path.join(problem_folder, "testcases_stock.txt")
                    f = open(testcase_path, 'w', encoding="utf-8", newline='')
                    f.write(dos2unix(content))
                    f.close()
                    CLog.info(f"Testcases saved to '{testcase_path}'")

            if problem.testcases_sample:
                with open(os.path.join(template_path, 'testcases.txt.j2'), encoding="utf-8") as file_:
                    template = Template(file_.read())
                    for i, t in enumerate(problem.testcases_sample):
                        t.name = "Test #%02d" % (i + 1)
                    content = template.render(testcases=problem.testcases_sample)
                    testcase_path = os.path.join(problem_folder, "testcases_sample.txt")
                    f = open(testcase_path, 'w', encoding="utf-8", newline='')
                    f.write(dos2unix(content))
                    f.close()
        elif format == "themis":
            CLog.info("Writing testcases in themis format")
            with ZipFile(os.path.join(problem_folder, 'testcases.zip'), 'w') as zip:
                if problem.testcases_sample:
                    for i, t in enumerate(problem.testcases):
                        folder = "SampleTest%02d" % (i + 1)
                        zip.writestr(os.path.join(folder, 'DATA.INP'), dos2unix(t.input))
                        zip.writestr(os.path.join(folder, 'DATA.OUT'), dos2unix(t.output))
                testcases = []
                if problem.stock_testcases:
                    testcases += problem.stock_testcases
                if problem.testcases:
                    testcases += problem.testcases
                if testcases:
                    for i, t in enumerate(testcases):
                        folder = "Test%02d" % (i + 1)
                        zip.writestr(os.path.join(folder, 'DATA.INP'), dos2unix(t.input))
                        zip.writestr(os.path.join(folder, 'DATA.OUT'), dos2unix(t.output))
                        # os.makedirs(folder, exist_ok=True)
                        # folder = os.path.join(problem_folder, "Test%02d" % (i + 1))
                        # with open(os.path.join(folder, 'DATA.INP'), 'w', encoding="utf-8") as fi:
                        #     fi.write(dos2unix(t.input))
                        # with open(os.path.join(folder, 'DATA.OUT'), 'w', encoding="utf-8") as fo:
                        #     fo.write(dos2unix(t.output))

        else:
            CLog.error(f"Saving testcases to format `{format}` not supported yet.")

    @staticmethod
    def generate_testcases(problem_folder, testcase_count=20,
                           programming_language=None, overwrite=False):
        """

        @param problem_folder:
        @param testcase_count:
        @param programming_language: py | cpp | pas, None = auto detect
        @return:
        """
        from ucode.services.dsa.problem_service import ProblemService
        # problem: Problem = ProblemService.load(problem_folder)
        meta, folders = ProblemService.detect_problem_code_in_folder(problem_folder)
        # print(problem)
        print(json.dumps(folders))
        print(json.dumps(folders['solutions']))
        print(json.dumps(folders['test_generators']))
        # return
        input_generator_files = folders['test_generators']
        solution_files = folders['solutions']

        if not input_generator_files:
            CLog.error("No input generator file found!")
            return
        if not solution_files:
            CLog.error("No solution file found!")
            return

        input_generator_file = input_generator_files[0]
        solution_file = solution_files[0]
        if programming_language:
            prefered_generators = [v for v in input_generator_files if v.endswith(programming_language)]
            if prefered_generators:
                input_generator_file = prefered_generators[0]
            else:
                CLog.warn(f"No input generator file found for preferred language `{programming_language}`,"
                          f"Using `{input_generator_file}` instead.")

            prefered_solutions = [v for v in solution_files if v.endswith(programming_language)]
            if prefered_solutions:
                solution_file = prefered_solutions[0]
            else:
                CLog.warn(f"No solution file found for preferred language `{programming_language}`,"
                          f"Using `{input_generator_file}` instead.")

        input_generator_file = os.path.abspath(input_generator_file)
        solution_file = os.path.abspath(solution_file)
        print(input_generator_file, solution_file, sep='\n')

        testcases = []
        for i in range(1, testcase_count + 1):
            _input, input_compile_time, input_time = TestcaseService.execute(input_generator_file, _input=str(i),
                                                                             timeout=180)
            # print("input:", _input)
            _output, compile_time, output_time = TestcaseService.execute(solution_file, _input=_input.strip())

            testcases.append(TestCase(input=_input, output=_output))
            print("Testcase #%02d generated! Input generation compile time: %.2fs, "
                  "input generation running time: %.2fs, solution compile time: %.2f, solution running time: %.2fs"
                  % (i, input_compile_time, input_time, compile_time, output_time))

        if meta['testcase_format']:
            problem = Problem()
            problem.testcases = testcases
            TestcaseService.save_testcases(problem_folder, problem, meta['testcase_format'])
        return testcases

    @staticmethod
    def read_file(file_path, file, file_reader=None, ):
        try:
            data = file.read().decode("utf-8")
            return data
        except UnicodeDecodeError as exception:
            if file_reader:
                file = file_reader.open(file_path)
            else:
                file = open(file_path, 'r')
            file.seek(exception.start)
            exception.reason = f'File {file.name} corrupted at: {file.read()[:100]}...'
            raise exception
        except Exception as exception:
            # todo unknown exception
            raise exception


if __name__ == "__main__":
    # folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\hanoi_2020_2021_thcs\\p4_itable"
    # # folder = "../../../problems/p4_itable"
    # T = TestcaseService.generate_testcases(problem_folder=folder, format="ucode",
    #                                        programming_language="cpp",
    #                                        testcase_count=10)
    # print("testcases:", len(T))

    # testcase_file = "D:\\projects\\freecontest\\beginner\\Beginner Free Contest 11 (01-09-2019)" \
    #                 "\\Bộ test (01-09-2019)\\AVERAGE.zip"
    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\sample_testcases.zip"
    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\ucode\testcases.txt"
    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\ucode\testcases.zip"
    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\ucode"
    testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\themis"
    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\themis\sample_testcases.zip"
    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\themis\sample_testcases_nested.zip"
    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\themis\sample_testcases_nested_dif_case.zip"
    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\hackerrank\testcases_hackerrank.zip"
    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\hackerrank\testcases_hackerrank_nested.zip"
    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\cms"
    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\cms\BANK.zip"
    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\cms\cms_sample_nested.zip"

    testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\bridge.zip"
    testcases = TestcaseService.load_testcases(testcase_file)
    print(testcases)
    print(len(testcases))
    print(testcases[0])
    print(testcases[1])




import codecs
import csv
import os


def remove_if_exists(file_path: str):
    if os.path.exists(file_path):
        os.remove(file_path)


def write_file(file_path: str, content, write_mode: str = 'a', is_list=False):
    with codecs.open(file_path, write_mode, 'utf-8') as test_cycle:
        if is_list:
            for c in content:
                test_cycle.write(str(c) + '\n')
        else:
            test_cycle.write(content)
        test_cycle.close()


def write_list_simple_object_to_csv(file_path: str, list_object: list):
    with open(file_path, 'w', ) as csvfile:
        writer = csv.writer(csvfile)
        writer.writerow(['Name', 'Link'])
        for player in list_object:
            writer.writerow([player.name, player.link])



from locust import HttpUser, task, between, constant

from other.common.database.variable.storage import Storage
from other.common.database.variable.ucode_student_user_token import UCODE_STUDENT_USER_TOKEN


class Tester(HttpUser):
    wait_time = constant(1000000)
    token = None
    key = None
    user_id = None
    submission_id = None

    def on_start(self):
        # token = UCODE_STUDENT_USER_TOKEN[random.randint(0, len(UCODE_STUDENT_USER_TOKEN) - 1)]
        # self.client.headers.update({'access-token': token})
        Storage.CURRENT_TOTAL_USER = Storage.CURRENT_TOTAL_USER + 1
        while True:
            self.client.headers.update({'access-token': UCODE_STUDENT_USER_TOKEN[Storage.CURRENT_TOTAL_USER]})
            user_info_res = self.client.get("/api/users/info", name='user_info')
            if user_info_res.status_code == 200:
                self.token = UCODE_STUDENT_USER_TOKEN[Storage.CURRENT_TOTAL_USER]
                self.key = "test_user_draft_" + UCODE_STUDENT_USER_TOKEN[Storage.CURRENT_TOTAL_USER]
                self.user_id = user_info_res.json()['data']['id']
                contest_detail_res = self.client.get(f"/api/contests/gio-lap-trinh-mixed", name='contest_detail')
                if not contest_detail_res.json()['data']['current_submission']:
                    self.client.post(f"/api/learning/lesson/5090/submit-quiz",
                                     json={'list_answers': [], 'draft': True}, name='submit_draft')
                    contest_detail_res = self.client.get(f"/api/contests/gio-lap-trinh-mixed", name='contest_detail')
                self.submission_id = contest_detail_res.json()['data']['current_submission']
                user_data_draft_res = self.client.get(f"/api/user-data/draft?key={self.key}&user_id={self.user_id}",
                                                      name='user_draft')
                return
            else:
                Storage.CURRENT_TOTAL_INVALID_USER = Storage.CURRENT_TOTAL_INVALID_USER + 1

    @task
    def run_contest(self):
        # self.client.post(f"/api/learning/lesson/5090/submit-quiz",
        #                  json={'list_answers': [], 'draft': True}, name='submit_draft')

        # contest_detail_res = self.client.get(f"/api/contests/gio-lap-trinh-mixed", name='contest_detail')
        # self.submission_id = contest_detail_res.json()['data']['current_submission']

        # user_data_draft_res = self.client.get(f"/api/user-data/draft?key={self.key}&user_id={self.user_id}",
        #                                       name='user_draft')

        # user_data_draft2_res = self.client.post(f"/api/user-data/draft",
        #                                         json={"key": "test_user_draff" + self.key,
        #                                               "value": "{\"46675\":{\"data\":1,\"status\":\"waiting\"},\"46691\":{\"data\":1,\"status\":\"waiting\"},\"46693\":{\"data\":1,\"status\":\"waiting\"},\"46694\":{\"data\":1,\"status\":\"waiting\"},\"46696\":{\"data\":3,\"status\":\"waiting\"},\"46697\":{\"data\":1,\"status\":\"waiting\"}}"},
        #                                         name='user_draft')
        #
        # submit_draft_res = self.client.post(f"/api/learning/lesson/5090/submit-quiz",
        #                                     json={"list_answers": [{"question_id": 46675, "user_answer": "1"},
        #                                                            {"question_id": 46691, "user_answer": "1"},
        #                                                            {"question_id": 46693, "user_answer": "1"},
        #                                                            {"question_id": 46694, "user_answer": "1"},
        #                                                            {"question_id": 46696, "user_answer": "3"},
        #                                                            {"question_id": 46697, "user_answer": "1"}],
        #                                           "draft": True,
        #                                           "submission_id": self.submission_id}, name='submit_draft')
        # if submit_draft_res.status_code != 200:
        #     print(self.token, submit_draft_res.status_code)

        self.client.post(f"/api/learning/lesson/5090/submit-quiz",
                         json={"list_answers": [{"question_id": 46675, "user_answer": "1"},
                                                {"question_id": 46691, "user_answer": "1"},
                                                {"question_id": 46693, "user_answer": "1"},
                                                {"question_id": 46694, "user_answer": "1"},
                                                {"question_id": 46696, "user_answer": "3"},
                                                {"question_id": 46697, "user_answer": "1"}], "draft": False,
                               "submission_id": self.submission_id}, name='submit')



from os import listdir
from os.path import isdir, isfile


def get_files_in_folder(root_folder: str):
    return [root_folder + '/' + item for item in listdir(root_folder)]


def get_all_folders_in_folder(root_folder: str, result: list = []):
    if isdir(root_folder) and any(isfile(root_folder + '/' + item) for item in listdir(root_folder)):
        result.append(root_folder)
        return result
    if isdir(root_folder) and listdir(root_folder):
        for subdir in listdir(root_folder):
            if isdir(root_folder + '/' + subdir):
                get_all_folders_in_folder(root_folder + '/' + subdir, result)
    return result



from dataclasses import dataclass, field
from typing import List, Union, Optional

from dataclasses_json import dataclass_json
from ucode.helpers.clog import CLog
from ucode.services.common import find_section


def try_cast(v, _type):
    try:
        v = _type(v)
    except ValueError:
        pass
    except TypeError:
        pass

    return v


@dataclass_json
@dataclass
class InputVariable:
    name: str = ""
    data_type: str = "" # integer, float, string, array.integer, array.array.float...
    min_value: str = None # Number or variable name
    max_value: str = None # Number or variable name
    available_values: str = None # set of value {...} or string ""
    min_value_inclusive: bool = True
    max_value_inclusive: bool = True

    # Number or variable name for 1D-array
    # list of 2 value (Number or variable name): min_len & max_len for string, always inclusive those two boundaries
    # list of number or variable name for multidimension arrays, each for 1 dimension
    # None for others
    length: Union[Union[int, str], List[Union[int, str]]] = None
    item_var: '__class__' = None    # input variable cua item (for string, array, multidimension array)

    def parse(self, _raw: str):
        _type, _name = _raw.split()
        # print(_type, " - ", _name)
        if _type in ['float', 'double', 'long double']:
            self.data_type = 'float'
            self.name = _name
        elif _type in ['int', 'integer']:
            self.data_type = 'integer'
            self.name = _name
        elif _type in ['str', 'string']:
            self.data_type = 'string'
            self.name = _name
        elif _type.startswith('array'):
            self.data_type = _type
            i = _name.find("[")
            self.name = _name[:i]
            sizes = _name[i:].replace("[", " ").replace("]", " ").split()
            for j in range(len(sizes)):
                if sizes[j].isdigit():
                    sizes[j] = int(sizes[j])
                else:
                    sizes[j] = f"`{sizes[j]}`"
            if len(sizes) == 1:
                self.length = sizes[0]
            else:
                self.length = sizes
        else:
            CLog.error("Not supported data type `%s` for variable `%s`" % (_type, _name))

    def parse_constraint(self, var_raw: str, L, H, is_min_inclusive, is_max_inclusive, available_values):
        """

        @param var_raw: a | s.length | a[i] | s[i] | a[i][j]
        @param L:
        @param H:
        @param is_min_inclusive:
        @param is_max_inclusive:
        @param available_values:
        @return:
        """
        if var_raw == self.name:
            self.min_value = L
            self.max_value = H
            self.min_value_inclusive = is_min_inclusive
            self.max_value_inclusive = is_max_inclusive
            self.available_values = available_values
        elif var_raw == self.name + ".length": # for string
            self.length = (L, H)
        elif var_raw.index("[") > 0:
            self.item_var = InputVariable()
            if var_raw.endswith(".length"):
                self.item_var.length = (L, H)
            else:
                self.item_var.min_value = L
                self.item_var.max_value = H
                self.item_var.min_value_inclusive = is_min_inclusive
                self.item_var.max_value_inclusive = is_max_inclusive
                self.item_var.available_values = available_values
        else:
            CLog.error("Constraint not supported: " + var_raw)

@dataclass_json
@dataclass
class InputLine:
    index: int = 0
    raw: str = ""
    description: str = ""
    repeat: Union[int, str] = 1  # number or variable name
    variables: List[InputVariable] = field(default_factory=list)

    def parse(self, _raw: str):
        self.raw = _raw
        raw_vars = _raw.lstrip("- ")
        dash_i = raw_vars.find("-")
        if dash_i >= 0:
            self.description = raw_vars[dash_i+1:].strip()
            raw_vars = raw_vars[:dash_i]
        raw_vars = raw_vars.strip()

        L = 0
        while L < len(raw_vars):
            R = L
            if raw_vars[L] == '`':
                R = L+1
                while R < len(raw_vars) and raw_vars[R] != '`':
                    R += 1
                var = raw_vars[L+1:R]
                if var.startswith("["):
                    self.repeat = var.lstrip("[").rstrip("]")
                    print("repeat: ", self.repeat)
                else:
                    input_var = InputVariable()
                    input_var.parse(var)
                    self.variables.append(input_var)
            L = R+1
        # print(raw_vars)


@dataclass_json
@dataclass
class InputSpecification:
    output_spec: str = ""
    multiple_testcase: bool = False
    multiple_testcase_var: InputVariable = None
    input_lines: List[InputLine] = field(default_factory=list)

    def find_var_by_name(self, name: str) -> Optional[InputVariable]:
        if self.multiple_testcase_var and self.multiple_testcase_var.name == name:
            return self.multiple_testcase_var
        for input_line in self.input_lines:
            for v in input_line.variables:
                if v.name == name:
                    return v
        return None

    def parse_constraint(self, constraint: str):
        var, limit = constraint.lstrip().lstrip("-").strip().split(":")
        var = var.strip("`")
        # print(var, limit)
        i = 0
        while i<len(var) and var[i] != "." and var[i] != "[":
            i += 1
        var_name = var[:i]
        # print("name:", var_name)

        limit: str = limit.strip()
        is_min_inclusive = limit.startswith("[")
        is_max_inclusive = limit.endswith("]")
        available_values = None
        L = H = None
        if limit.startswith("[") or limit.startswith("("):
            L, H = [v.strip() for v in limit.strip("[]()").split(",")]
        elif limit.startswith("{") and limit.endswith("}"):
            available_values = eval(limit)
        else:
            available_values = limit.lstrip("'").rstrip("'").lstrip('"').rstrip('"')
        # print(L, H, is_min_inclusive, is_max_inclusive, available_values)

        if var_name == "Subtasks":
            return

        input_var = self.find_var_by_name(var_name)
        print(f"found input var name `{var_name}`:", input_var)
        if not input_var:
            CLog.error(f"Variable not defined in input, but found in constraints: ``{var_name}")
            return

        # support integer and array.integer and s.length
        if "integer" in input_var.data_type or "float" in input_var.data_type or "length" in var:
            L = try_cast(L, float)  # call this first for integer value, to support "1e9"
            H = try_cast(H, float)
            if "integer" in input_var.data_type or "length" in var:
                L = try_cast(L, int)
                H = try_cast(H, int)

        input_var.parse_constraint(var, L, H, is_min_inclusive, is_max_inclusive, available_values)

    def parse(self, input_desc: str, constraints: str, output_desc: str):
        # input parsing
        lines = input_desc.strip().splitlines()
        if not lines:
            CLog.error("Empty input")
            return None
        if not (lines[0].startswith("[//]: # ") and "Auto" in lines[0]):
            CLog.error("This works for auto generate input format only. "
                       "Input description should start with line: [//]: # (Auto)")
            return None

        indices, content = find_section("^-", lines, start_index=1, until_pattern="^-", skip_section_header=False)
        # print(*lines, sep="\n")

        i = 0
        for idx in indices:
            i += 1
            input_line = InputLine(index=i)
            raw_lines = " ".join([s.strip() for s in content[idx]])
            input_line.parse(raw_lines)
            if input_line.description.startswith("Multitest"):
                self.multiple_testcase = True
                self.multiple_testcase_var = input_line.variables[0]
            else:
                self.input_lines.append(input_line)
            # print(input_line)

        # constraints parsing
        lines = constraints.strip().splitlines()
        if not lines:
            CLog.error("Empty constraints")
            return None

        indices, content = find_section("^-", lines, start_index=0, until_pattern="^-", skip_section_header=False)
        print(indices, content, sep="\n")
        for idx in indices:
            self.parse_constraint(content[idx][0])

        # output parsing
        self.output_spec = output_desc
        return self


def test_generate_data():
    _input_desc = """
[//]: # (Auto)
- `integer T` - Multitest
- `integer n` `integer m` - Số lượng phần tử $n$, 
số lượng thao tác $m$
- `int x` - số nguyên trong tập {2, 3, 5, 8} 
- `string name` - Tên
- `array.string subject[3][x]` - Tên
- `array.integer a[n]` - Các phần tử $a_i$ của mảng $a$, 
cách nhau bởi dấu cách
- `array.array.float b[n][m]` - Ví dụ ma trận 
- `double L` `double R` `[m]` - Cận trái và phải của một 
trong số $m$ thao tác 
"""

    _output_desc = """
- output is freedom for now
- `boolean YES|NO` - Kết quả bài toán
- `integer`- Kết quả bài toán
- `float(3)`- Kết quả bài toán, với 3 chữ số sau dấu phẩy
"""

    _constraints = """
- `T`: [1, 100]
- `n`: (2, 1e6]
- `m`: (2, 1e4)
- `x`: {2, 3, 5, 8}
- `a[i]`: [1, 1e18)
- `name.length`: [8, 8]
- `name[i]`: "qwertyuiopasdfghjklzxcvbnm"
- `subject[i].length`: [10, 20]
- `b[i][j]`: (1.1, 2.1)
- `L`: (1, 10000) 
- `R`: (`L`, 10000)
- Subtasks:
    - 40%: 
        - `T`: [1, 10]
        - `n`: [1, 100]
    - 30%: 
        - `n`: [100, 10000]
    - 40%: 
        - `n`: [10000, 1e6]
"""

    input_spec = InputSpecification()
    input_spec.parse(_input_desc, _constraints, _output_desc)

    print("Spec: ")
    print(input_spec.to_json())


if __name__ == "__main__":
    # folder = "D:\\projects\\ucode\\_problem_dev\\big_mod_py"
    # T = TestcaseService.generate_testcases(problem_folder=folder, format="ucode", testcase_count=20)
    # print("testcases:", len(T))

    test_generate_data()




# coding=utf-8
import json
import logging

__author__ = 'ThucNC'

import configparser
import mimetypes
import os
from typing import List, Union

import requests
from ucode.helpers.clog import CLog
from ucode.helpers.config import Config
from ucode.helpers.misc import make_slug, dos2unix
from ucode.models.problem import Problem, TestCase
from ucode.models.question import Question, QuestionType
from ucode.services.dsa.problem_service import ProblemService

_logger = logging.getLogger(__name__)


class UCode:
    def __init__(self, env="prod", domain="ucode.vn"):
        self.s = requests.session()
        self.env = env
        self.token = None
        self.tenant = domain
        self._headers = {}
        if env == "prod":
            self.api_base_url = "https://ucodeapis.com/api"
            self._headers['Referer'] = f'https://{domain}'
        else:
            self.api_base_url = "https://dev.ucodeapis.com/api"
            self._headers['Referer'] = 'https://dev.ucode.vn'
        self.load_config()

    def load_config(self):
        config = Config.get_instance().load()
        tenant_config = config.get(self.tenant)
        if not tenant_config or not tenant_config.get("credentials"):
            return
        credentials = tenant_config.get("credentials")
        if not credentials or not credentials.get(self.env):
            return
        self.token = credentials[self.env].get("access-token")
        self._headers['access-token'] = self.token

    def login(self, email, password):
        url = self.get_api_url(f"/users/sign-in")
        print(url)
        data = {
            "email": email,
            "password": password
        }
        response = self.s.post(url, json=data, headers=self._headers)
        print(self._headers)
        res_json = json.loads(response.text)
        if response.status_code != 200:
            CLog.error(f"Login Failed: {res_json['message']}")
            return None

        self.token = res_json["data"]

        config = Config.get_instance().load()
        if self.tenant not in config:
            config[self.tenant] = {}
        if "credentials" not in config[self.tenant]:
            config[self.tenant]["credentials"] = {}
        config[self.tenant]["credentials"][self.env] = {
            "env": self.env,
            "access-token": self.token
        }

        Config.get_instance().save(config)
        CLog.info(f"User `{email}` logged in successfully")

    def get_logged_in_user(self):
        self.load_config()
        if not self.token:
            CLog.error("Please login with ucode email and password first!")
            return None
        url = self.get_api_url("/users/info")
        print(url)
        print(self._headers)

        response = self.s.get(url, headers=self._headers)
        print(response.text)
        res_json = json.loads(response.text)

        if response.status_code == 401:
            CLog.error(f"Access token expired, please re-login with valid username and password.")
            return None

        print(f"Logged in `{self.env}` env with user `{res_json['data']['email']}`!")

    def upload_file(self, file_path, upload_folder):
        url = self.get_api_url(f"/general/upload-file?folder={upload_folder}")
        # url = "https://httpbin.org/post"
        with open(file_path, 'rb') as f:
            file_name = os.path.basename(file_path)
            print(file_name)
            # content_type = "application/binary"
            content_type = mimetypes.guess_type(file_path)[0]
            print("Content-Type:", content_type)
            files = {'file': (file_name, f, content_type)}
            response = self.s.post(url, files=files, headers=self._headers)
            # file_content = open(file_path, 'rb').read()
            # data = {'file': file_content}
            # response = self.s.post(url, data=data, headers=self._headers)
            # print("url:", url)
            # print("payload:", json.dumps(data))
            print("status code", response.status_code)
            print(response.text)
            if response.ok:
                return response.json().get("data")
        return None

    def upload_and_replace_problem_resource(self, problem: Problem, upload_folder="problem_resources"):
        if not problem.resource_files:
            return

        for resource in problem.resource_files:
            filename = os.path.basename(resource)
            print("resource:", filename, resource)
            remote_url = self.upload_file(resource, upload_folder)
            if not remote_url:
                CLog.error(f"Cannot upload file `{resource}`")
            ProblemService.replace_image_urls(problem, filename, remote_url)

    def get_api_url(self, path):
        return self.api_base_url + path

    def add_question_to_quiz(self, question_ids: List[int], quiz_id: int):
        url = self.get_api_url(f"/lesson-item/{quiz_id}/add-question")
        payload = {
            "question_ids": question_ids
        }

        response = self.s.post(url, json=payload, headers=self._headers)
        print("status_code:", response.status_code)
        print(url)
        print(response.text)
        res = response.json()
        # print(res)
        if res['success']:
            print(f"questions(s) successfully added to quiz {quiz_id}:", question_ids)
        else:
            CLog.error("Cannot add questions to quiz:" + json.dumps(res))

    def create_problem(self, problem: Union[Question, Problem, str],
                       score=10, xp=10, lang='vi', statement_format="markdown", question_type='code',
                       status="published", visibility="unlisted", request_approval=False):
        """
        :param problem:
        :param score:
        :param xp: -1 để tự động tính theo độ khó
        :param lang:
        :param statement_format:
        :param question_type: multiple_choice, short_answer, code, turtle, sport
        :return:
        """
        if isinstance(problem, str):
            problem: Problem = ProblemService.load(problem, load_testcase=True)

        if isinstance(problem, Problem):
            self.upload_and_replace_problem_resource(problem)

            ucoin = xp
            if ucoin < 0:
                ucoin = int(round(50 * problem.difficulty**1.5, -2))
            if ucoin <= 0:
                ucoin = 100
            data = {
                "name": problem.name,
                "slug": problem.slug if problem.slug else make_slug(problem.name),
                "type": question_type,
                "statement": problem.statement,
                "statement_format": statement_format,
                "input_desc": problem.input_format,
                "output_desc": problem.output_format,
                "constraints": problem.constraints,
                "compiler": "python",
                "statement_language": lang,
                "score": score,
                "difficulty": problem.difficulty,
                "status": status,
                "visibility": visibility,
                "ucoin": ucoin,
                # "source": problem.src_url if problem.src_url else "",
                "source_detail": problem.src_url if problem.src_url else "",
                'tags': problem.tags
            }
            if problem.editorial:
                data['hint'] = json.dumps({
                    "text": problem.editorial,
                    "text_type": "markdown"
                })
            if problem.solutions:
                solution_text = ""
                for solution in problem.solutions:
                    solution_text += f"```{solution['lang']}\n"
                    solution_text += f"{solution['code']}\n"
                    solution_text += f"```\n\n\n"

                data['solutions'] = json.dumps({
                    "text": solution_text,
                    "text_type": "markdown"
                })

            if request_approval:
                data['approval_status'] = 'waiting_for_approval'
            # if problem.solution:
            #     data["solution"] = problem.solution
        elif isinstance(problem, Question):
            question: Question = problem
            data = {
                "name": question.src_name,
                "type": question.type.value or question_type,
                "statement": question.statement,
                "statement_format": question.statement_format or statement_format,
                "statement_language": lang,
                "score": score,
                "status": status,
                "visibility": visibility,
                "ucoin": xp,
                "constraints": "base_question" if question.base_question else
                                               ("sub_question" if question.sub_question else "")
            }

            new_option_format = True
            # new_option_format = False
            # if not isinstance(question.options, list):
            #     new_option_format = True
            # else:
            #     for i, option in enumerate(question.options):
            #         if not isinstance(option, QuestionOption):
            #             # CLog.error("Option not valid:")
            #             # print(question.to_json())
            #             new_option_format = True
            #             break
            #         if not option.content:
            #             new_option_format = True
            #             break
            #         data[f"option{i + 1}"] = option.content
            #         if option.is_correct:
            #             data['answer'] = f"{i + 1}"

            question_json = json.loads(question.to_json())
            if question.solution:
                data['solution'] = question.solution
            elif question.solutions:
                data['solutions'] = json.dumps(question_json['solutions'])

            if question.hint:
                data['hint'] = json.dumps(question_json["hint"])

            if question.statement_media:
                data['statement_media'] = json.dumps(question_json["statement_media"])

            if question.option_display:
                data['option_display'] = json.dumps(question_json["option_display"])

            if question.source:
                data['source'] = json.dumps(question_json["source"])

            if new_option_format:
                data['options'] = json.dumps(question_json["options"])

            if question.answer is not None:
                data['answers'] = question.answer
            if question.type == QuestionType.SHORT_ANSWER:
                print("data:", data)
        else:
            raise Exception(f"Unsupported problem type {type(problem)}")

        # if lesson_id:
        #     url = self.get_api_url(f"/lesson-item/{lesson_id}/question")
        # else:
        url = self.get_api_url(f"/question")

        response = self.s.post(url, json=data, headers=self._headers)
        # print("url:", url)
        # print("payload:", json.dumps(data))
        print("status code", response.status_code)
        print(response.text)
        res = response.json()
        print(res)
        if res['success']:
            question_id = res['data']['id']
            print("question_id:", question_id)
        else:
            raise Exception("Cannot create question:" + json.dumps(res))

        if isinstance(problem, Problem):
            # upload testcase
            # for i, testcase in enumerate(problem.testcases):
            #     self.upload_testcase(question_id, testcase, is_sample=i<2)
            self.upload_testcases(question_id, ProblemService.join_testcases(problem))

        if isinstance(problem, Problem):
            if problem.translations:
                for tran_lang, tran_problem in problem.translations.items():
                    CLog.info(f"Creating translation {tran_lang} for question #{question_id}...")
                    data = {
                        "name": tran_problem.name,
                        "type": "code",
                        "root_question_id": question_id,
                        "statement": tran_problem.statement,
                        "statement_language": tran_lang,
                        "input_desc": tran_problem.input_format,
                        "output_desc": tran_problem.output_format,
                        "constraints": tran_problem.constraints,
                        "status": status,
                        "visibility": visibility,

                    }
                    # print("url:", url)
                    # print("payload:", json.dumps(data))

                    response = self.s.post(url, json=data, headers=self._headers)
                    print("status code", response.status_code)
                    # print(response.text)
                    res = response.json()
                    if res['success']:
                        tran_question_id = res['data']['id']
                        print("translated question_id:", tran_question_id)
                    else:
                        raise Exception("Cannot create question:" + json.dumps(res))

        return question_id

    def create_problems(self, problems: List[Union[Question, Problem, str]], score=10, xp=100, lang="vi", quiz_id=None):
        question_ids = []
        for problem in problems:
            q_id = self.create_problem(problem=problem, score=score, xp=xp, lang=lang)
            question_ids.append(q_id)

        if quiz_id:
            self.add_question_to_quiz(quiz_id=quiz_id, question_ids=question_ids)

        return question_ids

    def upload_testcases(self, problem_id, testcases: List[TestCase], score=10, sample_score=0):
        url = self.get_api_url(f"/question/{problem_id}/add-testcases")
        testcase_data = []
        for i, testcase in enumerate(testcases):
            t = {
                "name": testcase.name,
                "explanation": testcase.explanation,
                "input": dos2unix(testcase.input),
                "output": dos2unix(testcase.output),
                "score": sample_score if testcase.sample else score,
                "is_sample": testcase.sample
            }
            testcase_data.append(t)
        payload = {
            "testcases": testcase_data
        }

        response = self.s.post(url, json=payload, headers=self._headers)
        print("url:", url)
        print("status_code:", response.status_code)
        print(response.text)
        res = response.json()
        # print(res)
        if res['success']:
            print(len(testcases), "testcase(s) upload successfully")
        else:
            CLog.error("Cannot create testcase:" + json.dumps(res))

    def upload_testcase(self, problem_id, testcase: TestCase, is_sample=True, score=10):
        url = self.get_api_url(f"/question/{problem_id}/testcase")
        data = {
            "name": testcase.name,
            "explanation": testcase.explanation,
            "input": dos2unix(testcase.input),
            "output": dos2unix(testcase.output),
            "score": 0 if is_sample else score,
            "is_sample": bool(is_sample)
        }

        response = self.s.post(url, json=data, headers=self._headers)
        print(response.status_code)
        res = response.json()
        print(res)
        if res['success']:
            testcase_id = res['data']['id']
            print("testcase_id:", testcase_id)
        else:
            CLog.error("Cannot create testcase:" + json.dumps(res))

    @staticmethod
    def read_credential(credential_file):
        config = configparser.ConfigParser()
        config.read(credential_file)
        if not config.has_section('UCODE'):
            CLog.error(f'Section `UCODE` should exist in {credential_file} file')
            return None, None
        if not config.has_option('UCODE', 'api_url') or not config.has_option('UCODE', 'token'):
            CLog.error(f'api_url and/or token are missing in {credential_file} file')
            return None, None

        api_url = config.get('UCODE', 'api_url')
        token = config.get('UCODE', 'token')

        return api_url, token

    def create_chapter(self, course_id, chapter_name, slug=None, parent_id=None,
                       status="draft", _type="chapter", is_free=True):
        if not slug:
            slug = make_slug(chapter_name)

        data = {
            "parent_id": parent_id if parent_id else 0,
            "item_type": _type,
            "name": chapter_name,
            "is_preview": False,
            # "content_type": "video",
            "is_free": is_free,
            "slug": slug,
            "status": status
        }

        url = self.get_api_url(f"/curriculum/{course_id}/course-items")

        response = self.s.post(url, json=data, headers=self._headers)

        print(response.status_code)
        res = response.json()
        print(res)
        if res['success']:
            return res['data']['id']
        else:
            raise Exception("Cannot create chapter:" + json.dumps(res))

    def create_lesson_item(self, course_id, chapter_id, lesson_name, slug=None,
                           desciption="", content="",
                           type="video", video_url="",
                           file_url="",
                           file_type="pdf",
                           status="published", is_free=True, visibility="unlisted", ucoin=100):
        if not slug:
            slug = make_slug(lesson_name)

        data = {
            "parent_id": chapter_id,
            "item_type": "lesson_item",
            "name": lesson_name,
            "is_preview": False,
            "content_type": type,
            "is_free": is_free,
            "slug": slug,
            "status": status
        }

        url = self.get_api_url(f"/curriculum/{course_id}/course-items")

        response = self.s.post(url, json=data, headers=self._headers)
        print(response.status_code)
        res = response.json()
        # print(res)
        if not res['success']:
            raise Exception("Cannot create course item for lesson:" + json.dumps(res))

        course_item_id = res['data']['id']
        print("course_item_id:", course_item_id)
        lesson_item_id = res['data']['lesson_item_id']
        print("lesson_item_id:", lesson_item_id)
        # if type == "video":
        if video_url or content or desciption:
            url = self.get_api_url(f"/lesson-item/{lesson_item_id}")
            data = {
                "description": desciption,
                "video_url": video_url,
                "file_url": file_url,
                "file_type": file_type,
                "content": content,
                "name": lesson_name,
                "content_format": "markdown",
                "slug": slug,
                "is_free": is_free,
                "is_preview": False,
                "visibility": visibility,
                "ucoin": ucoin,
                "status": status,
            }

            response = self.s.put(url, json=data, headers=self._headers)
            print(response.status_code)
            # print(response.text)
            res = response.json()
            print(res)
            if not res['success']:
                raise Exception("Cannot lesson item:" + json.dumps(res))
        return lesson_item_id


def create_chapters_mc1():
    ucode = UCode("dev")

    course_id = 7
    for w in range(1, 13):
        chapter_name = "Thử thách %02d" % w
        chapter_id = ucode.create_chapter(course_id=course_id, chapter_name=chapter_name)


def create_problem(env="dev", domain="ucode.vn"):
    # problems = ProblemService.read_all_problems_v1("/home/thuc/projects/ucode/weekly-algorithm-problems/week13",
    #                                                translations=["vi"], load_testcase=True)
    # problem_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\quan_ba_dinh_2020_2021_thcs_round2\\p03_capdoixung"
    problem_folder = "D:\\projects\\ucode\\weekly-algorithm-problems\\competitions\\teko\\impossible_check_in"
    problem = ProblemService.load(problem_folder, load_testcase=True)
    problems = [(problem_folder, problem)]

    ucode = UCode(env=env, domain=domain)

    print(len(problems))
    for i, (problem_folder, problem) in enumerate(problems):
        print(i+1, problem.name)
        # print(problem.tags)
        # print(problem.translations['vi'].statement)
        print("Testcases: ", len(problem.testcases))
        # res = ucode.create_problem(problem=problem, lang="en", xp=-1, status="draft")
        # id = ucode.create_problem(problem=problem, xp=-1, request_approval=True)
        # print("question created with id:", id)
        # break


# def upload_all_problems_in_folder():
#     lession_id = 4046
#     # lession_id = 0 # public problem
#     base_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\BaDinh_2021_9"
#     problems = ProblemService.read_all_problems(base_folder, nested_folder=False)
#     print("Total problems:", len(problems))
#
#     # ucode = UCode("https://dev-api.ucode.vn/api", "26fd9211ce59375f5f01987bb868d170")
#     ucode = UCode("https://api.ucode.vn/api", "88a6973b6288aa09ba01767e8bd4ab94")
#
#     for i, (problem_folder, problem) in enumerate(problems):
#         problem = ProblemService.load(problem_folder, load_testcase=True)
#         print(i + 1, problem.name)
#         # ucode.create_problem(lesson_id=lession_id, problem=problem, xp=200)
#         print("testcases:", len(problem.testcases), len(problem.testcases_sample))
#         ucode_id = ucode.create_problem(lesson_id=lession_id, problem=problem, xp=-1)
#         print("uCode problem created: ", ucode_id)


def create_problems_to_lesson_ucode(env="dev", domain="ucode.vn"):
    # base_folder = "D:\\projects\\ucode\\repl_backup\\ThucNguyen1\\py1-1-lesson-6-7-if"

    # base_folder = "/home/thanh/Ucode/basic-problems/problem_set_01"
    # base_folder = "D:\\projects\\ucode\\basic_problems\\hoc_vong_2\\secondary-school\\hinh_chu_nhat"
    # base_folder = "D:\\projects\\ucode\\basic_problems\\math-coding-train\\secondary-school\\so_chinh_phuong"
    # base_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\quan_ba_dinh_2020_2021_thcs_round2"
    # base_folder = "D:\\projects\\ucode\\weekly-algorithm-problems\\competitions\\hsg-hn-2020_new"
    # base_folder = "D:\\projects\\ucode\\repl_backup\\itteacher-k1\\quiz9_turtle"
    # base_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\hanoi_2020_2021_thcs"
    # base_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\vinhphuc_2020_2021_thcs"
    # base_folder = "D:\\projects\\ucode\\repl_backup\\intro3"
    # base_folder = "D:\\projects\\ucode\\basic_problems\\hour-of-code-round2"
    # base_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\competitive_programing_topics\\maths\\fundamentals\\gcd_lcm_new"
    # base_folder = "D:\\projects\\freecontest\\beginner_formatted\\Beginner Free Contest 27 (15-01-2021)"
    # base_folder = "D:\\projects\\ucode\\competitive_programing_topics\\Maths\\Modulo & GCD - LCM\\Modular Arithmetic"
    # base_folder = "D:\\projects\\freecontest\\formatted\\Free Contest 97"

    # base_folder = r"D:\projects\ucode\weekly-algorithm-problems\competitions\teko-hcm"
    # base_folder = r"D:\projects\freecontest\mock_tests\mock01"
    # base_folder = r"D:\projects\ucode\competitive_programing_topics\tests\sample_tests\thcs_testTHT"
    base_folder = r"D:\projects\ucode\dsa-problem-bank\lamcherry\2021-09\thcs_tests\test_02"

    # base_folder = "D:\\projects\\ucode\\competitive_programing_topics\\Data_Structures\\Heap_PriorityQueue"
    problems = ProblemService.read_all_problems(base_folder, nested_folder=False)
    # problems = ProblemService.read_all_problems_v1(base_folder, nested_folder=False)

    # prob_folder = r"D:\projects\ucode\dsa-problem-bank\lamcherry\2021-09\thpt_tests\test_05\p04_bookshelves"
    # problems = [(prob_folder, ProblemService.load(prob_folder))]

    lesson_id = 23465
    ucode = UCode(env=env, domain=domain)
    #

    for i, (problem_folder, problem) in enumerate(problems):
        problem = ProblemService.load(problem_folder, load_testcase=True)
        # problem = ProblemService.load_v1(problem_folder, load_testcase=True, translations=['vi'])
        print(i + 1, problem.name, "testcases:", len(problem.testcases), ", sample test:", len(problem.testcases_sample))
        # print("Solution:", json.dumps(problem.solutions))
        # print("Editorial:", problem.editorial)
        # print("Constraint: ", problem.constraints)

        prob_id = ucode.create_problem(problem=problem, xp=-1, score=100)
        CLog.info(f"Created problem: {prob_id}")
        ucode.add_question_to_quiz(quiz_id=lesson_id, question_ids=[prob_id])
        print(f"Problem uploaded with id #{prob_id} and added to quiz {lesson_id}")

    print("Total problems:", len(problems))


def reupload_testcases():
    ucode = UCode("prod")
    problem_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\hanoi_2020_2021_thcs\\p4_itable"
    # problem_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\vinhphuc_2020_2021_thcs\\p3_weightloss"
    problem: Problem = ProblemService.load(problem_folder, load_testcase=True)

    question_id = 45490
    ucode.upload_testcases(question_id, ProblemService.join_testcases(problem))
    #
    print(len(problem.testcases))
    print(len(problem.testcases_sample))


def test_login():
    # ucode = UCode(env="prod", domain="teko.ugrow.vn")
    # ucode = UCode(env="dev", domain="teko.ugrow.vn")
    ucode = UCode(env="prod", domain="ucode.vn")
    # ucode.login("teko@ugrow.vn", "123456")
    # ucode.login("gthuc.nguyen@gmail.com", "ngoaho85")
    # ucode.login("gthuc.nguyen@gmail.com", "12345678")
    ucode.get_logged_in_user()


def teko_bb():
    bbs = """Product	06/03/2020	BB#1: Women in Tech	Dương Thu Hương		https://docs.google.com/presentation/d/1j56LrIp-VCgzBGuNb84WHfVrCcpG5XJOBF7rdhR14hY/edit
Product	10/12/2020	BB #23: Product Development Process	Hoàng Anh Duy	https://drive.google.com/file/d/1Lu8LzVpcl-REB_I5I1iEbGabIV_ihz9c/view	https://docs.google.com/document/d/13gnHsu9_5jU2cHAw940_inHXJ8YVbEXdPNBfNax_k0Y/edit
Product	11/03/2021	BB #29: How do we design metric for Product Managers	Nguyễn Viết Tuấn		https://drive.google.com/drive/folders/1iC512FI3HsfBLKCLbdaxe_1spjQXAgw-?usp=sharing
Product	25/02/2021	BB #27: How we build product at Teko - Project Timeline Management Tips	Nguyễn Viết Tuấn	https://drive.google.com/file/d/1SvvORUtwF_W4AJBnPnkDctBqVS15AsRo/view	
Product	25/03/2021	BB #31: Business Process: How do we design a Business Process for a Product Manager?	Nguyễn Viết Tuấn		https://docs.google.com/spreadsheets/d/12_Yl23tfqgK_QCVSAIwdG9t37IP3aB2vcMlc8tCsIBs/edit#gid=0
Product	08/04/2021	BB #33: UX Design Lesson Learned	Mai Thanh Bình	https://drive.google.com/file/d/1tD3mWc5Y0VFJEeO9zs9cZWah4UZ-i50N/view	https://docs.google.com/presentation/d/1XpX8a-r4A4SmE0szYIYBmHfr0gGBpKhFcx46ab20Y0o/edit
Product	29/04/2021	BB #36: Why most technical products fail	Mai Thanh Bình	https://drive.google.com/file/d/1x7bfPoZg8bCwsQULHYluoViKujcq7Onc/view	https://docs.google.com/presentation/d/1Im3LYi1nZb92Ot5GErlfr0RvB4T_i_nLfW3CMGUUtis/edit
Engineering	26/05/2020	BB#2: Golang	Nguyễn Xuân Dũng		https://drive.google.com/open?id=1CV3kO0MTGEqEx29eLJRVIPpQtHPGjyLn
Engineering	28/05/2020	BB#3: gRPC	Trần Duy Thịnh		
Engineering	03/06/2020	BB#4: Monorepo	Trần Đức Hiếu	https://drive.google.com/file/d/1g3QfY1NYbHcsZIaaqQDzDcCqbdaOHMm7/view	https://drive.google.com/file/d/1Zb0kAgnRbBITjTQ4Y_dHEJzxqUsbF1-P/view
Engineering	16/07/2020	BB#6: "Operation First Mobile Layout"	Vũ Minh Tâm	https://drive.google.com/file/d/1Fbip8XfG77hetX4qFNV9268Xb5woZnn1/view	https://docs.google.com/presentation/d/1nuHr9F9qDxSb3MdzXXIaLrsKrPXrIDEjIOV4gybNPn8/edit#slide=id.g8bf5102b19_0_248
Engineering	23/07/2020	BB#7: "Microservices and events"	Trần Tuấn Anh	https://drive.google.com/file/d/125BT6GpDR3lW-_KYrasgoa2auT597qE2/view	https://docs.google.com/presentation/d/1HqVG1ULy6oqUGx3P1aFcy5G8EA0hqs6Zz89iEVnU06I/edit#slide=id.g82939b2d8f_0_37
Engineering	30/07/2020	BB#8: "Building containers: How many ways are too many?"	Trần Tuấn Anh	https://drive.google.com/file/d/1oWDFvN8zPcoLTLvSughfQvv3urM-WQQK/view	https://drive.google.com/file/d/18sja-mb0leEGy7b8xGBwHPhatCKqACJ6/view?usp=sharing
Engineering	06/08/2020	BB#9: Machine Learning in Production	Jason Tsai	https://drive.google.com/file/d/1_SfgysyM9-p3E0GwIUilmkOnd9BwDvLO/view	https://docs.google.com/presentation/d/1xAoXywFQot4R44u3mwVijHx3XMLw3aKP86Py_QtboMA/edit
Engineering	13/08/2020	BB#10: ETL	Hoàng Thị Loan	https://drive.google.com/file/d/19F-vEX1yrkR0NUMxx8mfFVG5SrTkt2YM/view	https://docs.google.com/presentation/d/1LIjK-z8df-st5Zf5J_aPpvgCmc1VLrhhbMwOkXtWu_Y/edit
Engineering	20/08/2020	BB#11: Redis	Trần Tuấn Anh	https://drive.google.com/file/d/1SlFG1InrUOouTJOMCpUtMSScVeenHmZZ/view	https://docs.google.com/presentation/d/1PDsvi8DbzfBddQdx3MgtTErgpVxO0r879T7xzz8VyeA/edit#slide=id.g82939b2d8f_0_37
Engineering	27/08/2020	BB#12-1: Doc Design	Jason Tsai	https://drive.google.com/file/d/1P7956TwtT6Dy3J7eIE__fgeGDhDaOxz5/view	https://docs.google.com/presentation/d/1Gk7YDTWFIGxdHJ0HNFm5uXum_cFERaSvHogaSPlwZdY/edit#slide=id.p
Engineering	03/09/2020	BB#12-2: Code Reviews	Trần Tuấn Anh	https://drive.google.com/file/d/1r08yefAVj9NSTg4xeEjX5BNJTgZ8xZ1e/view	https://docs.google.com/presentation/d/1Gk7YDTWFIGxdHJ0HNFm5uXum_cFERaSvHogaSPlwZdY/edit#slide=id.p
Engineering	17/09/2020	BB #14: Development process (for e-commerce projects)	Vũ Minh Tâm, Đặng Thị Phương	https://drive.google.com/file/d/1-ieKUp32LASz2DNqP3v6uFuKMU-85s2P/view	https://docs.google.com/presentation/d/1-f_fvfmS_y5YjwSqyPx5MolJiB0Fj6vKwQ46hG68HsE/edit#slide=id.g98131b7036_0_9
Engineering	24/09/2020	BB #15: Mock Design Review	Tạ Ngọc Vân Thoa	https://drive.google.com/file/d/1s4RVmhTkpBam-aTLAVR4zWbMoaUkCK6w/view	
Engineering	15/10/2020	BB #16: Building React Native Bridge for VNShop App	Lê Vũ Huy	https://drive.google.com/file/d/1TxSCuAae4hOVUkyDMi7SLYshttY6YtMA/view	https://docs.google.com/presentation/d/1cMXxOK2axbUsQSzY1HqG8fejv8vu67QC2__iHQxuEcQ/edit
Engineering	29/10/2020	BB #17: Golang Best Practices	Nguyễn Quốc Anh	"https://drive.google.com/file/d/1q28n1g2DzcTeWY_D_170E4lDasNYDElD/view https://drive.google.com/file/d/1Ql_9VUUE304zkYV1P8T97TSUYb4zK0oc/view"	https://docs.google.com/presentation/d/1tyXReUJwqy6L4MMtB3mKobhNS9MKZwG5F-K1AR-34MI/edit
Engineering	05/11/2020	BB #18: State of Teko Engineering	Nguyễn Việt Tiến	https://drive.google.com/file/d/1xTEzAMiH9toK7AITNwm_FIu0H-drHkZR/view	https://docs.google.com/presentation/d/1CDaR9WVssb-1wQ3oEjYgxKDx6-Bbc8LAU0kJfuDafEY/edit
Engineering	19/11/2020	BB #20: Under the hood of Elasticsearch	Cao Mạnh Đạt	https://drive.google.com/file/d/1W0B-d8M_WMbcmpMtRrYDhV8-g4ErY7N_/view?usp=sharing	https://docs.google.com/presentation/d/1Fr2X4elikv1sP9kFK0Kzogs_Hx0dN4KEj15kiknUe7I/edit
Engineering	26/11/2020	BB #21: Terra Platform - How we make Super App at VNLife	Đinh Văn Tiến	"https://drive.google.com/file/d/1UJo7uIvqCa1zU0KnW7soIxjSokmqVY4U/view https://drive.google.com/file/d/1-5DP_CUfU0zPIAICyRSdpL18JrO0ONos/view"	https://docs.google.com/presentation/d/1-kxCb1tg5IgR8Bm8lmVy1YysfI4tUTArOntgMb9tpcc/edit#slide=id.g35f391192_00
Engineering	03/12/2020	BB #22: Engineering Excellence - Post-Mortem Guidelines	Jason Tsai	https://drive.google.com/file/d/1frKvuUKxUrOkLF0Hhjeg1LXNPiZefOmH/view	https://docs.google.com/document/d/1G7UGJsccjTtRr0N4maGI2PsUbnstnBkhUXYhug5ESDc/edit
Engineering	17/12/2020	BB #24: Tech Practices | API Improvement Proposals by Google | E1/S1: Standard methods	Trần Phong Phú	https://drive.google.com/file/d/17Ykl8en180rgaAi7hcZz71-bSlcCylvl/view	https://docs.google.com/presentation/d/1CEMTZgLi67x96eylEGsEXOubiWg28FVC2uFmZsx5JKg/edit
Engineering	24/12/2020	BB #25: Kubernetes Cluster in the Active/Standby Role	Huỳnh Phạm Sony	https://drive.google.com/file/d/1rW-mtu9Isx6WFzeohuAbofaD8YSe1eJK/view	
Engineering	14/01/2021	BB #26: : TerraBus - Event bus for Terra Platform	Nguyễn Bá Tú, Nguyễn Xuân Tùng	"https://drive.google.com/file/d/1S7hlPyS1dE2XhIhelyzgqnPOQ8mWJtXh/view https://drive.google.com/file/d/11YH7ywZX4qdn09zOCpNxrDeEHMrAES1F/view"	
Engineering	18/03/2021	BB #30: Unit Test The Painless	Trần Phong Phú	https://drive.google.com/file/d/1izQN9eSb460AC-APRKXWXwH_Uugm3JNJ/view	https://docs.google.com/presentation/d/1mfQXGnWhY0WcP4Fw6LjTRvD-xfJP3DRF1Ef1-I3FQjc/edit?usp=sharing
Engineering	01/04/2021	BB #32: Streaming Ingestion - things learned	Cao Mạnh Đạt	https://drive.google.com/file/d/1m9TwT6TiGWeMcLx3EYzii-jYAAk8uYKz/view	https://drive.google.com/file/d/11oqqMlk5BjO98D-eSc2ZRqtMeVdAJSJR/view
Engineering	22/04/2021	BB #35: Ceph - Distributed storage for Kubernetes	Đặng Minh Dũng	https://drive.google.com/file/d/1xRq9Fi90-jPsgFccHJuTypnKv6b76JSC/view	https://docs.google.com/presentation/d/1meHpDaEHdEYjfvipJo8N4XPcN8AWThS2MBFOVxnd_CM/edit
Engineering	06/05/2021	BB #37: High-level Omni-channel Architecture Design	Jason Tsai	https://drive.google.com/file/d/1tFHN8h6g-yss8yuCrpJrfxKXZRWik17S/view	https://docs.google.com/presentation/d/1_WC0al4teKuT1gWFGOa_9yZrAfU4TJwqSOiY7Lr3vdE/edit?usp=sharing
Engineering	27/05/2021	BB #39: Coding and Reviewing Best Practices	MinhPT	https://drive.google.com/file/d/1h8aPo-FBlBs7jFActCVIlhPY1VrouTTk/view	https://docs.google.com/presentation/d/1WiUrUa5MD8BR_nGsmx0W0F_My4ppn9arbzAzy0GY6YQ/edit
Engineering	10/06/2021	BB #41: Feature Flags & Best Practices	TriTM	https://drive.google.com/file/d/1Lt9MoBr8WQChQVvfmxEaSs4jvXMKsSPs/view	https://docs.google.com/presentation/d/1SZITJDVZMb9m_sAiY4i5kLC5Gigcg00zf3a82uNAQ6Y/edit#slide=id.g81b07784d9_0_212
Engineering	17/06/2021	BB #42: Event-driven Architecture	PhúTP		https://docs.google.com/presentation/d/10VsN5-cFvVpwckPCN96fORIiXfHPV5YTx6Cm8niIwFo/edit#slide=id.g82939b2d8f_0_37
Life Style	04/03/2021	BB #28: Weight Management	Lê Thu Thảo, Lê Thị Thanh Nhàn		https://drive.google.com/file/d/1RMsC0dzFBlNO12wYiEhjEDSflHppyxB3/view
Life Style	15/04/2021	BB #34: Personal Time Management	Ôn Như Bình		https://docs.google.com/presentation/d/19RpQz4nFlOetQOTDlL13P4KAga23zOnULv34OHOBAas/edit#slide=id.g82939b2d8f_0_37
Life Style	13/05/2021	BB #38: Personal Financial Management	BinhMT	https://drive.google.com/file/d/1c-b10PrqHkhD_wOhdXEDvXK4wuwZBQ7Q/view	https://docs.google.com/presentation/d/1YR4syEgLWD_5lY5XSuAEqiKGpn5G1RKhpMUjw5kYduw/edit
Skill	03/06/2021	BB #40: What should you write in Performance Reviews	Jason Tsai	https://drive.google.com/file/d/1SAJ7HAo9nIS4qidBK_G5LP-07OKfmn8D/view	https://docs.google.com/presentation/d/1t91PUVgUAwog2N9sQaxzs_Zkf1N_h1Pq9sS2oGmvVmM/edit
UX/UI	25/06/2020	BB#5: Design System Workshop	UX/UI Design Team		
UX/UI	12/11/2020	BB #19: How to use Teko Design System for PO an BA	Trần Thành Đạt, Phạm Mạnh Hùng	https://www.figma.com/file/SpKK4ZTpP5NygVto1KHZfQ/Design-System-Guide?node-id=0%3A1	https://www.figma.com/file/Ssd74IcfyJYZTt673CDHmI/DESIGN-SYSTEM-LIBRARY-WORKSHOP"""

    track_ids = {
        "Product": 9640,
        "Engineering": 9641,
        "UX/UI": 9642,
        "Life Style": 9643,
        "Skill": 9644,
    }
    course_id = 669

    track_ids = {
        "Product": 9453,
        "Engineering": 9454,
        "UX/UI": 9455,
        "Life Style": 9456,
        "Skill": 9457,
    }
    course_id = 648
    ucode = UCode(env="dev", domain="teko.ugrow.vn")

    for bb in bbs.split("\n"):
        track, date, title, host, video_url, slide_url = bb.split("\t")
        print(track, host, slide_url)
        print(track_ids[track])
        chapter_id = ucode.create_chapter(course_id=course_id, chapter_name=title, parent_id=track_ids[track],
                                          status="published")
        # chapter_id = 9482
        print("Chapter created:", chapter_id)
        lesson_id = ucode.create_lesson_item(course_id=course_id,
                                 chapter_id=chapter_id,
                                 lesson_name=title,
                                 desciption=title,
                                 type="lecture",
                                 status = "published",
                                 content=f"Date: {date}\n\nHost: {host}")
        print("Lesson info created:", lesson_id)
        lesson_id = ucode.create_lesson_item(course_id=course_id,
                                             chapter_id=chapter_id,
                                             lesson_name=title + " - Video",
                                             desciption=title,
                                             type="video",
                                             status="published",
                                             video_url=video_url,
                                             content=title + f" - Video\n\n{video_url}" if video_url else "There is no videos for this lesson.")
        print("Lesson video created:", lesson_id)
        lesson_id = ucode.create_lesson_item(course_id=course_id,
                                             chapter_id=chapter_id,
                                             lesson_name=title + " - Slide",
                                             desciption=title,
                                             type="file",
                                             file_url=slide_url,
                                             file_type="",
                                             status="published",
                                             content=title + f" - Slide\n\n{slide_url}" if slide_url else "There is no slide for this lesson.")
        print("Lesson slide created:", lesson_id)
        # break


if __name__ == "__main__":
    test_login()
    # teko_bb()
    # reupload_testcases()

    # create_problems_to_lesson_ucode("prod", domain="teko.ugrow.vn")
    create_problems_to_lesson_ucode("prod")

    # create_problem("dev")
    # create_problem("prod", domain="teko.ugrow.vn")
    # create_problem("prod", domain="ucode.vn")
    # upload_all_problems_in_folder()

    # lession_id = 4068
    # # lession_id = 0
    # # #
    # # # # ucode = UCode("https://dev-api.ucode.vn/api", "26fd9211ce59375f5f01987bb868d170")
    # ucode = UCode("https://api.ucode.vn/api", "88a6973b6288aa09ba01767e8bd4ab94")
    # # #
    # problem_folder = "D:\\projects\\ucode\\dsa-problems\\unsorted\\thuc\\beautiful_pairs"
    # problem: Problem = ProblemService.load(problem_folder, load_testcase=True)
    #
    # print(problem.name)
    # print(len(problem.testcases))
    # print(len(problem.testcases_sample))
    # print(problem.tags)
    # print(problem.resource_files)
    #
    # ucode_id = ucode.create_problem(lesson_id=0, problem=problem, xp=-1)
    # print("uCode problem created: ", ucode_id)
    # print(problem.statement)
    # print(problem.translations['en'].statement)


    # img_file = "C:/Users/thucnguyen/Downloads/FB Posts.png"
    # img_file = "C:/Users/thucnguyen/Downloads/z2154907105699_598b6d3a26a9aa817fc97c2c8079d261.jpg"
    # upload_url = ucode.upload_file(img_file, "thuc_test")
    # print(upload_url)
    # ucode_courses = {
    #     2: 42,
    #     3: 43,
    #     4: 44,
    #     5: 45,
    #     6: 46,
    #     7: 47,
    #     8: 48,
    #     9: 49,
    #     10: 50,
    #     11: 51
    # }
    # ucode_courses = {
    #     2: 46
    # }
    # for y, course_id in ucode_courses.items():
    #     iqsha(y, course_id)

    # create_chapters_mc1()

    # ucode = UCode("https://dev-api.ucode.vn/api", "72821b59462c5fdb552a049c1caed85c")
    # # problem_folder = "../../problems/domino_for_young"
    # problem_folder = "/home/thuc/projects/ucode/courses/course-py101/lesson2/c1_input/p13_chao_ban"
    # ucode_lesson_id = 172
    #
    # problem: Problem = ProblemService.load(problem_folder, load_testcase=True)
    # #
    # # print(problem)
    # #
    # print(len(problem.testcases))
    # for testcase in problem.testcases:
    #     print(testcase)

    # ucode.create_problem(lesson_id=172, problem=problem_folder)

    # beestar = Beestar()
    #
    # files = beestar.read_quizzes_files_from_folder(
    #     "/home/thuc/projects/ucode/content_crawler/data/beestar/*grade-4-math*_ans.html"
    # )
    # course_id = 17
    # # chappter_id = 413  # GT Math 2
    # # chappter_id = 630  # GT Math 5
    # chappter_id = 735
    # # chappter_id = 619  # problems
    #
    # ucode = UCode("https://dev-api.ucode.vn/api", "df12e0548fbba3e6f48f9df2b78c3df2")
    # # for file in files:
    # #     ucode.create_lesson_item_from_beestar_file(course_id=course_id, chapter_id=chappter_id, beestar_file=file)
    # file = "../../problems/bs/_bs_ans.html"
    # ucode.create_lesson_item_from_beestar_file(course_id=course_id, chapter_id=chappter_id, beestar_file=file)

    # problem_folder = "/home/thuc/projects/ucode/problemtools/problems/cs/Arcade_TheCore/p001_Intro_Gates__addTwoDigits"
    # problem = ProblemService.load(problem_folder, load_testcase=True)
    #
    # hr = HackerRank()
    # hr.login('thucngch', '15041985')
    # # prepare_testcases("/home/thuc/projects/ucode/problemtools/problems/cs/Arcade_TheCore/p001_Intro_Gates__addTwoDigits")
    # hr.upload_testcases(163975, "/home/thuc/projects/ucode/problemtools/problems/cs/Arcade_TheCore/p001_Intro_Gates__addTwoDigits/testcases_hackerrank.zip")
    # # hr_prob = hr.create_problem(problem)
    # # print(json.dumps(hr_prob))g
    #
    # "https://www.hackerrank.com/rest/administration/contests/46715/challenge?slug=add-two-digits&track_id=0&chapter_id=0&weight=100"
    #
    #
    # ucode = UCode("https://dev-api.ucode.vn/api", "df12e0548fbba3e6f48f9df2b78c3df2")
    # # print(problem)
    # # print(problem.name)
    # # print(len(problem.testcases))
    # #
    # # ucode.create_problem(lesson_id=None, problem=problem, question_type='code', lang="en")








import PySimpleGUI as sg

sg.theme('DarkAmber')   # Add a touch of color
# All the stuff inside your window.
layout = [  [sg.Text('Some text on Row 1')],
            [sg.Text('Enter something on Row 2'), sg.InputText()],
            [sg.Button('Ok'), sg.Button('Cancel')] ]

# Create the Window
window = sg.Window('Window Title', layout)
# Event Loop to process "events" and get the "values" of the inputs
while True:
    event, values = window.read()
    if event == sg.WIN_CLOSED or event == 'Cancel': # if user closes window or clicks cancel
        break
    print('You entered ', values[0])

window.close()


import codecs
import json
import re
import time

import requests

from ucode.models.problem import Problem
from ucode.services.dsa.problem_service import ProblemService

BASE_URL = "https://dev-api.ucode.vn/api"
ACCESS_TOKEN = "f0875cb3517009fee6c6962da5f9560b"


def _get_difficulty(num_of_prb, section):
    fixed = [
        [1, 73, 1.5, 2],
        [2, 108, 1.2, 1.2],
        [4, 113, 1.5, 1.5],
        [5, 300, 1.5, 2.5],
        [6, 418, 2, 3.5],
        [7, 564, 2, 3.5],
        [8, 610, 2, 3],
        [10, 669, 2, 3.5],
        [11, 685, 2, 3.5],
        [12, 714, 2, 3.5],
        [13, 745, 1, 2.5]]
    for score in fixed:
        if f"Chương {score[0]}" in section:
            pc = num_of_prb / score[1]
            return round(score[3], 1) if pc > 1 else round(score[2] + (score[3] - score[2]) * pc, 1)
    score = fixed[10]
    pc = num_of_prb / score[0]
    return round(score[3], 1) if pc > 1 else round(score[2] + (score[3] - score[2]) * pc, 1)


def download_source_code():
    rs = []
    file_path = "problems3.json"
    output_file_path = "problems3_with_source_code.json"
    with open(file_path, encoding="utf-8-sig") as json_file:
        prbs = json.load(json_file).get("Transformed by JSON-CSV.CO")
        count = 0
        for prb in prbs:
            count += 1
            prb = dict(prb)
            print(f"Downloading ...{count}")
            source_code = ''
            if prb.get("url").strip() != "n":
                code_res = requests.get(prb.get("url").strip() + "/raw.code")
                if code_res.status_code == 200:
                    source_code = code_res.text
            prb.update({"source_code": source_code})
            rs.append(prb)
    with codecs.open(output_file_path, 'a', 'utf-8') as out:
        out.write(json.dumps(rs))
        out.close()


def convert_to_ucode_problem_format():
    file_path = "problems3_with_source_code.json"
    with open(file_path, encoding="utf-8-sig") as json_file:
        prbs = json.load(json_file)
        num_of_prb = 0
        num_of_urs_prb = 0
        tags_lv0 = set()
        tags_lv1 = set()
        for prb in prbs:
            num_of_prb += 1
            problem = Problem(f"Bài tập số {num_of_prb}")
            problem.src_name = f"Bài tập số {num_of_prb}"
            problem.solution_lang = "c"
            # url and source_code
            if prb.get("url").strip() == "n":
                num_of_urs_prb += 1
                problem.src_url = ''
            else:
                problem.src_url = prb.get("url").strip()
            problem.solution = prb.get("source_code")
            # tag
            tags = prb.get("section").split("___")
            tag_lv0 = re.sub("Chương [0-9|:|\*|\(|\)| ]+", "", tags[0].strip()).lower()
            tags_lv0.add(tag_lv0)
            problem.tags.append(tag_lv0)
            if len(tags) > 1:
                tag_lv1 = tags[1].strip().lower()
                tags_lv1.add(tag_lv1)
                problem.tags.append(tag_lv1)
            # statement
            statement = re.sub("Bài [0-9|:|\*|\(|\)| ]+", "", prb.get("des").strip())
            statement[0].upper()
            problem.statement = statement
            # difficulty
            problem.difficulty = _get_difficulty(num_of_prb=num_of_prb, section=prb.get("section"))
            print(f"num {num_of_prb} {problem.difficulty}")
            ProblemService.save(problem=problem, base_folder="problem")
        json_file.close()
    print(f"{num_of_urs_prb} invalid problems/{num_of_prb} problems")
    print(f"{len(tags_lv0)} lv0_tag / {len(tags_lv1)} lv1_tag")
    print(tags_lv0)
    print(tags_lv1)


def call_api_create_snippet():
    file_path = "problems3_with_source_code.json"
    with open(file_path, encoding="utf-8-sig") as json_file:
        prbs = json.load(json_file)
        num_of_prb = 0
        for prb in prbs:
            num_of_prb += 1
            # if num_of_prb > 400:
            #     break
            # if num_of_prb < 397:
            #     continue
            # source_code
            source_code = prb.get("source_code")
            # tag
            problem_tags = []
            tags = prb.get("section").split("___")
            tag_lv0 = re.sub("Chương [0-9|:|\*|\(|\)| ]+", "", tags[0].strip()).lower()
            problem_tags.append(tag_lv0)
            if len(tags) > 1:
                tag_lv1 = tags[1].strip().lower()
                problem_tags.append(tag_lv1)
            # statement
            statement = re.sub("Bài [0-9|:|\*|\(|\)| ]+", "", prb.get("des").strip())
            statement[0].upper()
            # call api
            data = {
                "description": statement,
                "name": f"Bài tập số {num_of_prb}: {prb.get('section')}",
                "programming_language": "50",
                "source_code": source_code,
                "slug": f"bai-tap-so-{num_of_prb}",
                "tags": problem_tags,
                "input": "", "output": ""}
            res = requests.post(f"{BASE_URL}/snippet/", data=json.dumps(data),
                                headers={"content-type": "application/json", "charset": "UTF-8",
                                         "access-token": ACCESS_TOKEN})
            print(f"{num_of_prb}___{prb.get('url')}____{res.status_code}")
            time.sleep(0.05)


# download_source_code()
convert_to_ucode_problem_format()
# call_api_create_snippet()



import os

import typer
from pathlib import Path

from ucode.helpers.clog import CLog
from ucode.services.dsa.problem_service import ProblemService
from ucode.ucode.ucode_srv import UCode

app = typer.Typer()


@app.command(name="upload")
def upload_problem_to_ucode(
        problem_folder: Path = typer.Argument(".", help="Problem folder to upload, default to current folder.",
                                              exists=True, file_okay=False),
        lesson_id: int = typer.Option(None, "--lesson", "-l",
                                      help='Lesson ID to which the problem will be added to if specified'),
        server: str = typer.Option("ucode.vn", "--server", "-s",
                                   help='Server (domain) of site to upload, default to `ucode.vn`'),
        env: str = typer.Option("prod", "--env", "-e",
                                help='prod|dev for ucode environment'),
):
    problem = ProblemService.load(os.path.abspath(problem_folder), load_testcase=True)
    CLog.important(f"Loaded problem `{problem.name}` with {len(problem.testcases)} testcases, "
              f"and {len(problem.testcases_sample)} sample testcases")
    ucode = UCode(env, domain=server)
    prob_id = ucode.create_problem(problem=problem, xp=-1, score=100)
    CLog.info(f"Uploaded problem with id #`{prob_id}` to `{server}`")
    if lesson_id:
        ucode.add_question_to_quiz(quiz_id=lesson_id, question_ids=[prob_id])
        CLog.info(f"Problem #{prob_id} added to quiz #{lesson_id}")


@app.command(name="upload_all")
def upload_all_problems_to_ucode(
        folder: Path = typer.Argument(".", help="Folder contain all problems to upload, default to current folder.",
                                              exists=True, file_okay=False),
        lesson_id: int = typer.Option(None, "--lesson", "-l",
                                      help='Lesson ID to which the problem will be added to if specified'),
        server: str = typer.Option("ucode.vn", "--server", "-s",
                                   help='Server (domain) of site to upload, default to `ucode.vn`'),
        env: str = typer.Option("prod", "--env", "-e",
                                help='prod|dev for ucode environment'),
        check: bool = typer.Option(False, "--check", "-c",
                                       help='Check problems without real uploading'),
):
    problems = ProblemService.read_all_problems(folder, nested_folder=False)
    CLog.info(f"Founded {len(problems)} problem folders in `{folder}`")

    uploaded = []
    for i, (problem_folder, problem) in enumerate(problems):
        problem = ProblemService.load(os.path.abspath(problem_folder), load_testcase=True)
        CLog.important(f"Loaded problem #{i+1}: `{problem.name}` with {len(problem.testcases)} testcases, "
                  f"and {len(problem.testcases_sample)} sample testcases")
        if not check:
            ucode = UCode(env, domain=server)
            prob_id = ucode.create_problem(problem=problem, xp=-1, score=100)
            uploaded.append(prob_id)
            CLog.info(f"Uploaded problem with id #`{prob_id}` to `{server}`")
            if lesson_id:
                ucode.add_question_to_quiz(quiz_id=lesson_id, question_ids=[prob_id])
                CLog.info(f"Problem #{prob_id} added to quiz #{lesson_id}")

    CLog.important(f"Total problems uploaded: {len(uploaded)}")

    if check:
        CLog.important(f"Checking mode only, no problems uploaded")


@app.command(name="login")
def login(email: str = typer.Argument(..., help='ucode.vn email'),
          password: str = typer.Argument(..., help='ucode.vn password'),
          server: str = typer.Option("ucode.vn", "--server", "-s",
                                   help='Server (domain) of site to upload, default to `ucode.vn`'),
          env: str = typer.Option("prod", "--env", "-e",
                                   help='prod|dev for ucode environment'),
          ):
    ucode = UCode(env, domain=server)
    ucode.login(email, password)


if __name__ == "__main__":
    app()


import time

import redis
from locust import HttpUser, task, constant

from other.common.database.variable.storage import Storage


class Tester(HttpUser):
    wait_time = constant(5)
    token = None
    key = None
    user_id = None
    submission_id = None

    def on_start(self):
        # token = UCODE_STUDENT_USER_TOKEN[random.randint(0, len(UCODE_STUDENT_USER_TOKEN) - 1)]
        # self.client.headers.update({'access-token': token})
        redisClient = redis.StrictRedis(host='localhost',
                                        port=6379,
                                        db=0)
        while True:
            self.token = redisClient.lpop('tokens').decode('utf-8')
            self.client.headers.update({'access-token': self.token})
            user_info_res = self.client.get("/api/users/info", name='user_info')
            Storage.CURRENT_TOTAL_USER = Storage.CURRENT_TOTAL_USER + 1
            if user_info_res.status_code == 200:
                # self.token = UCODE_STUDENT_USER_TOKEN[Storage.CURRENT_TOTAL_USER]
                self.key = "test_user_draft_" + self.token
                self.user_id = user_info_res.json()['data']['id']
                contest_detail_res = self.client.get(f"/api/contests/gio-lap-trinh-mixed", name='contest_detail')
                if not contest_detail_res.json()['data']['current_submission']:
                    self.client.post(f"/api/learning/lesson/5090/submit-quiz",
                                     json={'list_answers': [], 'draft': True}, name='submit_draft')
                    contest_detail_res = self.client.get(f"/api/contests/gio-lap-trinh-mixed", name='contest_detail')
                self.submission_id = contest_detail_res.json()['data']['current_submission']
                user_data_draft_res = self.client.get(f"/api/user-data/draft?key={self.key}&user_id={self.user_id}",
                                                      name='user_draft')
                print(self.key, self.user_id , user_data_draft_res.json())
                return
            else:
                Storage.CURRENT_TOTAL_INVALID_USER = Storage.CURRENT_TOTAL_INVALID_USER + 1

    @task
    def run_contest(self):
        time.sleep(1000)
        # self.client.post(f"/api/learning/lesson/5090/submit-quiz",
        #                  json={'list_answers': [], 'draft': True}, name='submit_draft')

        # contest_detail_res = self.client.get(f"/api/contests/gio-lap-trinh-mixed", name='contest_detail')
        # self.submission_id = contest_detail_res.json()['data']['current_submission']

        # user_data_draft_res = self.client.get(f"/api/user-data/draft?key={self.key}&user_id={self.user_id}",
        #                                       name='user_draft')
        for i in range(25):
            user_data_draft2_res = self.client.post(f"/api/user-data/draft",
                                                    json={"key": self.key,
                                                          "value": "{\"46675\":{\"data\":1,\"status\":\"waiting\"},\"46691\":{\"data\":1,\"status\":\"waiting\"},\"46693\":{\"data\":1,\"status\":\"waiting\"},\"46694\":{\"data\":1,\"status\":\"waiting\"},\"46696\":{\"data\":3,\"status\":\"waiting\"},\"46697\":{\"data\":1,\"status\":\"waiting\"}}"},
                                                    name='user_draft')

            submit_draft_res = self.client.post(f"/api/learning/lesson/5090/submit-quiz",
                                                json={"list_answers": [{"question_id": 46675, "user_answer": "1"},
                                                                       {"question_id": 46691, "user_answer": "1"},
                                                                       {"question_id": 46693, "user_answer": "1"},
                                                                       {"question_id": 46694, "user_answer": "1"},
                                                                       {"question_id": 46696, "user_answer": "3"},
                                                                       {"question_id": 46697, "user_answer": "1"}],
                                                      "draft": True,
                                                      "submission_id": self.submission_id}, name='submit_draft')
            if submit_draft_res.status_code != 200:
                print(self.token, self.submission_id, submit_draft_res.status_code)
            time.sleep(1)

        res = self.client.post(f"/api/learning/lesson/5090/submit-quiz",
                               json={"list_answers": [{"question_id": 46675, "user_answer": "1"},
                                                      {"question_id": 46691, "user_answer": "1"},
                                                      {"question_id": 46693, "user_answer": "1"},
                                                      {"question_id": 46694, "user_answer": "1"},
                                                      {"question_id": 46696, "user_answer": "3"},
                                                      {"question_id": 46697, "user_answer": "1"}], "draft": False,
                                     "submission_id": self.submission_id}, name='submit')
        if res.status_code == 200:
            Storage.TOTAL_SUCCESS = Storage.TOTAL_SUCCESS + 1
        print(Storage.TOTAL_SUCCESS)



# coding=utf-8
import json
import logging

import requests

from ucode.helpers.misc import make_slug
from ucode.models.lesson import LessonType, QuizType
from ucode.models.question import QuestionType

__author__ = 'Dan'
_logger = logging.getLogger(__name__)


class QuestionService:
    def __init__(self, env='dev'):
        self._base_url, self._headers = self._get_ucode_info(env)

    @staticmethod
    def _get_ucode_info(env):
        if env == 'dev':
            return 'https://dev-api.ucode.vn/api', {'access-token': 'b269ec359ba979874f2d8180dcd5a638'}
        else:
            return 'https://api.ucode.vn/api', {'access-token': '371d83ff35571c02eb8609db2a48918a'}

    def create_contest(self, lesson_name, slug=None,
                       description="", content="", tags=None, is_graded_quiz=False, quiz_duration=None,
                       status="draft", is_free=True, visibility="public", ucoin=100):
        if not slug:
            slug = make_slug(lesson_name)
        data = {
            "name": lesson_name,
            "content_format": "markdown",
            "slug": slug,
            "is_free": is_free,
            "is_preview": False,
            "visibility": visibility,
            "ucoin": ucoin,
            "status": status,
            "type": LessonType.QUIZ.value,
            "quiz_type": QuizType.SUBMIT_ALL.value,
            "is_graded_quiz": is_graded_quiz
        }
        if quiz_duration:
            data['quiz_duration'] = quiz_duration
        if description:
            data['description'] = description
        if content:
            data['content'] = content
        if tags:
            data['tags'] = tags

        print(data)

        url = f"{self._base_url}/contests"
        print(url)
        response = requests.post(url, json=data, headers=self._headers)
        print(response.status_code)
        res = response.json()
        print(res)
        if not res['success']:
            raise Exception("Cannot create contest:" + json.dumps(res))
        return res['data'].get('id')

    def create_question(self, quiz_id, q_name, q_type, options, answers,
                        statement, statement_format="markdown", lang='vi',
                        status="draft", visibility="public", ucoin=10, score=10):
        data = {
            "name": q_name,
            "type": q_type,
            "statement": statement,
            "statement_format": statement_format,
            "statement_language": lang,
            "score": score,
            "status": status,
            "visibility": visibility,
            "ucoin": ucoin,
            "options": options,
            "answers": answers
        }

        url = f"{self._base_url}/lesson-item/{quiz_id}/question"
        response = requests.post(url, json=data, headers=self._headers)
        print(response.status_code)
        res = response.json()
        print(res)
        if not res['success']:
            raise Exception("Cannot create question:" + json.dumps(res))
        return res['data'].get('id')

    @staticmethod
    def parsing_file_markdown(in_file, out_file):
        quizzes = []
        with open(in_file, 'r', encoding="utf-8") as fi:
            questions = []
            quiz_title = ""
            line = fi.readline()
            while line:
                if line.startswith("## "):
                    if questions:
                        quizzes.append({'title': quiz_title,
                                        'questions': questions})
                    quiz_title = line[3:].split('::')[-1].strip()
                    questions = []
                    line = fi.readline()
                elif line.startswith("### "):
                    line = line[4:].split('::')
                    question_name = line[1].strip() if len(line) > 1 else ""
                    options = []
                    question_content = ""
                    line = fi.readline()
                    while line and not line.startswith("##"):
                        if line.startswith("- ["):
                            options.append({'text': line[5:].strip(),
                                            'is_correct': True if '[x]' in line or '[X]' in line else False,
                                            'text_type': 'markdown'})
                        else:
                            question_content += line
                        line = fi.readline()
                    question_ans = [str(i) for i in range(0, len(options)) if options[i].get('is_correct')]
                    if len(question_ans) == 1:
                        question_type = QuestionType.SINGLE_CHOICE.value
                    elif len(question_ans) > 1:
                        question_type = QuestionType.MULTI_CHOICE.value
                    else:
                        _logger.error(f'Cannot matching question type {question_name}')
                    questions.append({'name': question_name,
                                      'options': options,
                                      'answers': ','.join(question_ans),
                                      'type': question_type,
                                      'statement': question_content,
                                      'statement_format': 'markdown'})

                else:
                    line = fi.readline()

            if questions:
                quizzes.append({'name': quiz_title,
                                'questions': questions})

            print(json.dumps(quizzes))
            with open(out_file, 'w', encoding="utf-8", newline='') as f:
                f.write(json.dumps(quizzes))

    def create_quiz_from_json_file(self, file):
        with open(file, 'r', encoding="utf-8") as json_file:
            data = json.load(json_file)
            for quiz_info in data:
                quiz_id = self.create_contest(lesson_name=quiz_info.get('name'), tags=['Hour of Code'],
                                              is_graded_quiz=True, quiz_duration=3600)
                _logger.info(f"Created quiz {quiz_id}: {quiz_info.get('name')}")
                for info in quiz_info.get('questions'):
                    question_id = self.create_question(
                        quiz_id, q_name=info.get('name'), q_type=info.get('type'), statement=info.get('statement'),
                        options=json.dumps(info.get('options')), answers=info.get('answers'),
                        statement_format=info.get('statement_format'), status='published')
                    _logger.info(f"Created question {question_id}: {info.get('name')}")


if __name__ == "__main__":

    service = QuestionService('prod')
    # service.parsing_file_markdown('/home/thanh/Ucode/level1-sample.md',
    #                               '/home/thanh/Ucode/level1-sample.json')
    # service.parsing_file_markdown('/home/thanh/Ucode/level2-sample.md',
    #                               '/home/thanh/Ucode/level2-sample.json')
    # service.parsing_file_markdown('/home/thanh/Ucode/level3-sample.md',
    #                               '/home/thanh/Ucode/level3-sample.json')
    # service.parsing_file_markdown('/home/thanh/Ucode/level4-sample.md',
    #                               '/home/thanh/Ucode/level4-sample.json')
    # service.parsing_file_markdown('/home/thanh/Ucode/hour-of-code/contests/chinh_thuc/level3.md',
    #                               '/home/thanh/Ucode/hour-of-code/contests/chinh_thuc/level3.json')
    # service.parsing_file_markdown('/home/thanh/Ucode/hour-of-code/contests/chinh_thuc/level4.md',
    #                               '/home/thanh/Ucode/hour-of-code/contests/chinh_thuc/level4.json')
    # service.parsing_file_markdown('/home/thanh/Ucode/hour-of-code/contests/chinh_thuc/level1.md',
    #                               '/home/thanh/Ucode/hour-of-code/contests/chinh_thuc/level1.json')
    # service.parsing_file_markdown('/home/thanh/Ucode/hour-of-code/contests/chinh_thuc/level2.md',
    #                               '/home/thanh/Ucode/hour-of-code/contests/chinh_thuc/level2.json')
    # service.create_quiz_from_json_file('/home/thanh/Ucode/level1-sample.json')
    # service.create_quiz_from_json_file('/home/thanh/Ucode/level2-sample.json')
    # service.create_quiz_from_json_file('/home/thanh/Ucode/level3-sample.json')
    # service.create_quiz_from_json_file('/home/thanh/Ucode/level4-sample.json')
    # service.create_quiz_from_json_file('/home/thanh/Ucode/hour-of-code/contests/mixed.json')
    # service.create_quiz_from_json_file('/home/thanh/Ucode/hour-of-code/contests/chinh_thuc/level3.json')
    # service.create_quiz_from_json_file('/home/thanh/Ucode/hour-of-code/contests/chinh_thuc/level4.json')
    service.create_quiz_from_json_file('/home/thanh/Ucode/hour-of-code/contests/chinh_thuc/level1.json')
    service.create_quiz_from_json_file('/home/thanh/Ucode/hour-of-code/contests/chinh_thuc/level2.json')






# coding=utf-8
import json
import logging

__author__ = 'ThucNC'

import requests
from unidecode import unidecode

from ucode.helpers.clog import CLog

_logger = logging.getLogger(__name__)


class Vio:
    def __init__(self):
        self.s = requests.session()

    def login(self, username, password):
        url = "https://vio.edu.vn/login"
        data = {
            "username": username,
            "password": password
        }
        response = self.s.post(url, json=data)

        if response.status_code == 200:
            CLog.important(f"Login OK: {username} / {password}")
        elif response.status_code == 401:
            CLog.error(f"Login WRONG: {username} / {password}")
        else:
            CLog.error(f"Login FAIL: {username} / {password}")


if __name__ == "__main__":
    vio = Vio()
    schools = """Phan Bội Châu
Lê Anh Xuân
Lê Lợi
Đồng Khởi
Đặng Trần Côn
Võ Thành Trang
Hùng Vương
Tân Thới Hoà
Thoại Ngọc Hầu
Trần Quang Khải
Nguyễn Huệ
Hoàng Diệu
Tôn Thất Tùng"""

    # parttern = "c2%s-dp@hanoiedu.vn"
    parttern = "c2%s-tp@hcmedu.vn"
    for s in schools.split("\n"):
        name = unidecode(s).lower().replace(" ", "")
        vio.login(parttern % name, "123456789")



# coding=utf-8
import json
__author__ = 'ThucNC'

from ucode.services.common import find_section


class LessonService:
    @staticmethod
    def read_quiz_from_md(quiz_md):
        lines = quiz_md.splitlines()
        lines.append("")
        lines.append("")

        questions = []
        question_indices, question_contents = find_section('^(#{3})\s+.*', lines, until_pattern='^(#{3})\s+.*',
                                                           skip_section_header=False)
        # print(question_contents)
        for q_i, q_c in question_contents.items():
            print("########################")
            question_title = q_c[0][3:].strip()
            q_c = q_c[1:]
            print("question_title:", question_title)

            hint_solution_i, hint_solution_c = find_section('^(#{4}).*', q_c, skip_section_header=False)
            question_hint = question_solution = None
            if hint_solution_i:
                q_c = q_c[:hint_solution_i[0]]
                for s_i, s_c in hint_solution_c.items():
                    if "hint" in s_c[0].lower():
                        question_hint = "\n".join(s_c[1:])
                    if "solution" in s_c[0].lower():
                        question_solution = "\n".join(s_c[1:])

            option_pattern = '^\s*-\s*\[(.{0,1})\] .*'
            option_i, option_c = find_section(option_pattern, q_c, until_pattern=option_pattern,
                                              skip_section_header=False)
            question_options = []
            if option_i:
                question_statement = "\n".join(q_c[:option_i[0]])
                for option_lines in option_c.values():
                    o_i = option_lines[0].find("]")
                    is_correct = "[x]" in option_lines[0][:o_i+1].lower().replace(" ", "")
                    option_lines[0] = option_lines[0][o_i+1:].lstrip()
                    option_text = "\n".join(option_lines).strip()
                    # print("Option:", option_text, is_correct)
                    question_options.append({"content": option_text, "is_correct": is_correct})
            else:
                question_statement = "\n".join(q_c)

            # print("question_statement:", question_statement)
            # print("question_options:", question_options)
            # print("hint:", question_hint)
            # print("solution:", question_solution)
            questions.append({
                "title": question_title,
                "statement": question_statement,
                "options": question_options,
                "hint": question_hint,
                "solution": question_solution,
            })

        return questions


if __name__ == "__main__":
    quiz_file = r"D:\projects\ucode\ucode-cli\problems\quiz\quiz.md"
    with open(quiz_file, encoding="utf-8") as f:
        quiz_md = f.read()

    quiz = LessonService.read_quiz_from_md(quiz_md)
    print(json.dumps(quiz, indent=2))



from bs4 import BeautifulSoup

from ucode.helpers.clog import CLog
from ucode.models.question import Question, QuestionType, QuestionOption
from ucode.ucode.ucode_srv import UCode


class GForm:
    def __init__(self):
        pass

    def parse_html(self, html_file_path):
        file = open(html_file_path, mode='r')
        raw = file.read()
        file.close()
        soup = BeautifulSoup(raw, 'html.parser')
        title = soup.select("div.exportFormTitle")[0].text.strip()
        print(title)

        question_tags = soup.select("div[role='list'] div.freebirdFormviewerViewNumberedItemContainer[role='listitem']")
        # print(*question_tags, sep="\n")
        questions = []
        for question_tag in question_tags:
            question = Question()
            statement_tags = question_tag.select("div.exportItemTitle")
            image_tag = question_tag.select("img")
            image_url = None
            if image_tag:
                image_url = image_tag[0]["src"]
            if statement_tags:
                statement_tag = statement_tags[0]
            else:
                print("Error:", statement_tags)
                continue
            require_tag = statement_tag.select("span.freebirdFormviewerComponentsQuestionBaseRequiredAsterisk")
            if require_tag:
                require_tag[0].decompose()
            statement = statement_tag.text
            if not statement.strip():
                continue
            question.statement = statement
            question.statement_format = "markdown"
            if image_url:
                question.statement += "\n\n"
                question.statement += f"![image]({image_url})"

            note_tag = question_tag.select("div[role='note']")
            score = 1
            if note_tag:
                note = note_tag[0].text
                if note.endswith("points"):
                    score = int("".join([c for c in note if "0" <=c <= "9"]))

            question.score = score
            # class:
            # freebirdFormviewerComponentsQuestionRadioRoot
            # freebirdFormviewerComponentsQuestionCheckboxRoot
            # freebirdFormviewerComponentsQuestionSelectRoot
            # freebirdFormviewerComponentsQuestionTextRoot
            # freebirdFormviewerComponentsQuestionTextRoot
            if question_tag.select("div.freebirdFormviewerComponentsQuestionRadioRoot"):
                question.type = QuestionType.SINGLE_CHOICE
                option_tags = question_tag.select("div.freebirdFormviewerComponentsQuestionRadioRoot")[0]\
                    .select("div.freebirdFormviewerComponentsQuestionRadioChoice div.docssharedWizToggleLabeledContent")
                question.options = []
                for option_tag in option_tags:
                    option = QuestionOption(text=option_tag.text, text_type="plaintext")
                    question.options.append(option)
            elif question_tag.select("div.freebirdFormviewerComponentsQuestionCheckboxRoot"):
                question.type = QuestionType.MULTI_CHOICE
                option_tags = question_tag.select("div.freebirdFormviewerComponentsQuestionCheckboxRoot")[0] \
                    .select("div.freebirdFormviewerComponentsQuestionCheckboxChoice div.docssharedWizToggleLabeledContent")
                question.options = []
                for option_tag in option_tags:
                    option = QuestionOption(text=option_tag.text, text_type="plaintext")
                    question.options.append(option)
            elif question_tag.select("div.freebirdFormviewerComponentsQuestionSelectRoot"):
                question.type = QuestionType.SINGLE_CHOICE
                option_tags = question_tag.select("div.freebirdFormviewerComponentsQuestionSelectRoot")[0] \
                    .select("div.quantumWizMenuPaperselectOptionList div.quantumWizMenuPaperselectOption span.exportContent")
                question.options = []
                for option_tag in option_tags[1:]: # skip first item "Choose"
                    option = QuestionOption(text=option_tag.text, text_type="plaintext")
                    question.options.append(option)
            elif question_tag.select("div.freebirdFormviewerComponentsQuestionTextRoot"):
                question.type = QuestionType.SHORT_ANSWER
            else:
                CLog.error(f"Question not supported: {question_tag}")
            print(f"'{question.statement}'", question.type, question.score, question.options)

            questions.append(question)

        return title, questions


def create_quiz_ucode(title, questions):
    course_id = 648
    chapter_id = 9453
    ucode = UCode(env="dev", domain="teko.ugrow.vn")
    quiz_id = ucode.create_lesson_item(course_id=course_id, chapter_id=chapter_id, lesson_name=title,
                                       type="quiz")
    ucode.create_problems(questions, quiz_id=quiz_id)


if __name__ == '__main__':
    gf = GForm()
    # path = "D:\\projects\\ucode\\ucode-cli\\ucode\\oj\\gform\\Quizz_ Communication with Charisma.html"
    # path = "D:\\projects\\ucode\\ucode-cli\\ucode\\oj\\gform\\Sample quiz.html"
    path = "D:\\projects\\ucode\\ucode-cli\\ucode\\oj\\gform\\Sample quiz_result.html"
    title, questions = gf.parse_html(path)
    print(title, len(questions))

    create_quiz_ucode(title, questions)



class VioUser:
    # all
    vio_username: str = ''
    vio_password: str = ''
    full_name: str = ''
    ms_username: str = ''
    ms_password: str = ''
    gender: str = ''
    dob: str = ''
    address: str = ''
    # student
    student_class_name: str = ''
    student_code: str = ''
    parent_email: str = ''
    parent_phone_number: str = ''
    vio_parent_username: str = ''
    vio_parent_password: str = ''
    vio_parent_username_2: str = ''
    vio_parent_password_2: str = ''
    # teacher
    email: str = ''
    phone_number: str = ''
    identity_card_number: str = ''
    teacher_subject: str = ''
    teacher_class_name: str = ''
    # rank
    contest_count_wrong_answer: int = -1
    contest_count_correct_answer: int = -1
    contest_duration: int = -1
    contest_rank: int = -1
    # extra
    type: str = ''
    school_level: str = ''
    school_name: str = ''
    student_class: str = ''
    province_name: str = ''
    district_name: str = ''

    def __init__(self, **kwargs):
        for (key, value) in kwargs.items():
            if hasattr(self, key):
                setattr(self, key, value)

    @classmethod
    def get_header_str(cls):
        #     return "'vio_username','vio_password','full_name','ms_username','ms_password','gender','dob','address','student_class_name','student_code','parent_email','parent_phone_number','vio_parent_username', 'vio_parent_password', 'vio_parent_username_2', 'vio_parent_password_2', 'email', 'phone_number', 'identity_card_number', 'teacher_subject', 'teacher_class_name', 'contest_count_wrong_answer', 'contest_count_correct_answer', 'contest_duration', 'contest_rank', 'type', 'school_level', 'school_name','student_class', 'province_name', 'district_name'"
        return "vio_username,vio_password,full_name,ms_username,ms_password,gender,dob,address,student_class_name,student_code,parent_email,parent_phone_number,vio_parent_username, vio_parent_password, vio_parent_username_2, vio_parent_password_2, email, phone_number, identity_card_number, teacher_subject, teacher_class_name, contest_count_wrong_answer, contest_count_correct_answer, contest_duration, contest_rank, type, school_level, school_name,student_class, province_name, district_name"

    def __str__(self):
        return f"`{self.vio_username}`,`{self.vio_password}`,`{self.full_name}`,`{self.ms_username}`,`{self.ms_password}`,`{self.gender}`,`{self.dob}`,`{self.address}`,`{self.student_class_name}`,`{self.student_code}`,`{self.parent_email}`,`{self.parent_phone_number}`,`{self.vio_parent_username}`,`{self.vio_parent_password}`,`{self.vio_parent_username_2}`,`{self.vio_parent_password_2}`,`{self.email}`,`{self.phone_number}`,`{self.identity_card_number}`,`{self.teacher_subject}`,`{self.teacher_class_name}`,{self.contest_count_wrong_answer},{self.contest_count_correct_answer},{self.contest_duration},{self.contest_rank},`{self.type}`,`{self.school_level}`,`{self.school_name}`,`{self.student_class}`,`{self.province_name}`,`{self.district_name}`"
        # return f"{self.vio_username},{self.vio_password},{self.full_name},{self.ms_username},{self.ms_password},{self.gender},{self.dob},{self.address},{self.student_class_name},{self.student_code},{self.parent_email},{self.parent_phone_number},{self.vio_parent_username},{self.vio_parent_password},{self.vio_parent_username_2},{self.vio_parent_password_2},{self.email},{self.phone_number},{self.identity_card_number},{self.teacher_subject},{self.teacher_class_name},{self.contest_count_wrong_answer},{self.contest_count_correct_answer},{self.contest_duration},{self.contest_rank},{self.type},{self.school_level},{self.school_name},{self.province_name},{self.district_name}"



import codecs
import os
from pathlib import Path

from ruamel.yaml import YAML


class Config:
    __instance = None

    @staticmethod
    def get_instance():
        """ Static access method. """
        if not Config.__instance:
            Config()
        return Config.__instance

    def __init__(self):
        if Config.__instance:
            raise Exception("This class is a singleton!")
        else:
            Config.__instance = self

    def load(self):
        config_folder = os.path.join(Path.home(), ".ucode")
        config_file = os.path.join(config_folder, f"config.yaml")

        res = {}
        if os.path.isfile(config_file):
            with open(config_file, encoding="utf-8") as f:
                yaml = YAML(typ="safe")
                res = yaml.load(f)

        return res or {}

    def save(self, config):
        config_folder = os.path.join(Path.home(), ".ucode")
        os.makedirs(config_folder, exist_ok=True)
        config_file = os.path.join(config_folder, f"config.yaml")

        with codecs.open(config_file, "w", encoding="utf-8") as f:
            yaml = YAML()
            yaml.indent(offset=2)
            yaml.dump(config, f)

        return None


if __name__ == "__main__":
    config = Config.load_config()
    print(config)
    Config.save_config({"token": "asdfsa"})



import random
import string


def rand_int(vmin, vmax):
    return random.randint(vmin, vmax)


def rand_double(vmin, vmax):
    return random.uniform(vmin, vmax)


def rand_list(len, vmin, vmax, distinct = False):
    if not distinct:
        res = []
        for i in range(0, len):
            res.append(rand_int(vmin, vmax))
        return res
    else:
        return random.sample(range(vmin, vmax+1), len)


def rand_str(length, seed=string.ascii_lowercase):
    return ''.join(random.choices(seed, k=length))


# nhập vào một số nguyên dương duy nhất là số thứ tự của test case, bắt đầu từ 1
test_number = int(input())

# print("Generating testcase #%02d" % test_number)

if test_number == 1:
    # testcase tự soạn
    print(2, 5, 11)
elif test_number == 2:
    # testcase tự soạn
    print(3, 18132, 17)
else:
    if test_number <= 5:
        # test case ngẫu nhiên cho sub 1
        vmax = 100
    elif test_number <= 10:
        # test case ngẫu nhiên cho sub 2
        vmax = 100000
    elif test_number <= 15:
        # test case ngẫu nhiên cho sub 3
        vmax = 10000000
    else:
        # test case ngẫu nhiên cho sub 4
        vmax = 2 ** 31 - 1

    a = rand_int(0, vmax)
    b = rand_int(0, vmax)
    m = rand_int(1, 46340)

    print(a, b, m)
